#!/usr/bin/env python

#
# Part of NetDEF Topology Tests
#
# Copyright (c) 2017-2018 by
# Network Device Education Foundation, Inc. ("NetDEF")
# Modified by LabN Consulting, L.L.C.
#
# Permission to use, copy, modify, and/or distribute this software
# for any purpose with or without fee is hereby granted, provided
# that the above copyright notice and this permission notice appear
# in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NETDEF DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NETDEF BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
# OF THIS SOFTWARE.
#

"""
customize.py: Simple FRR/Quagga MPLS L3VPN test topology

             +---------+
             |    r1   |
             | 1.1.1.1 |                              PE Router
             +----+----+
                  | .1  r1-eth0
                  |
            ~~~~~~~~~~~~~
          ~~     sw0     ~~
          ~~ 10.0.1.0/24 ~~
            ~~~~~~~~~~~~~
                  |10.0.1.0/24
                  |
                  | .2  r2-eth0
             +----+----+
             |    r2   |
             | 2.2.2.2 |                              P router
             +--+---+--+
    r2-eth2  .2 |   | .2  r2-eth1
         ______/     \______
        /                   \
  ~~~~~~~~~~~~~        ~~~~~~~~~~~~~
~~     sw2     ~~    ~~     sw1     ~~
~~ 10.0.3.0/24 ~~    ~~ 10.0.2.0/24 ~~
  ~~~~~~~~~~~~~        ~~~~~~~~~~~~~
        |                 /    |
         \      _________/     |
          \    /                \
r3-eth1 .3 |  | .3  r3-eth0      | .4 r4-eth0
      +----+--+---+         +----+----+
      |     r3    |         |    r4   |
      |  3.3.3.3  |         | 4.4.4.4 |               PE Routers
      +-----------+         +---------+
 
"""

import os
import re
import pytest

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger
from lib.ltemplate import ltemplateRtrCmd

# Required to instantiate the topology builder class.
from mininet.topo import Topo

import shutil
CWD = os.path.dirname(os.path.realpath(__file__))
# test name based on directory
TEST = os.path.basename(CWD)

class ThisTestTopo(Topo):
    "Test topology builder"
    def build(self, *_args, **_opts):
        "Build function"
        tgen = get_topogen(self)

        # This function only purpose is to define allocation and relationship
        # between routers, switches and hosts.
        #
        # Create P/PE routers
        tgen.add_router('r1')
        for routern in range(2, 5):
            tgen.add_router('r{}'.format(routern))
        # Create a switch with just one router connected to it to simulate a
        # empty network.
        switch = {}
        switch[0] = tgen.add_switch('sw0')
        switch[0].add_link(tgen.gears['r1'], nodeif='r1-eth0')
        switch[0].add_link(tgen.gears['r2'], nodeif='r2-eth0')

        switch[1] = tgen.add_switch('sw1')
        switch[1].add_link(tgen.gears['r2'], nodeif='r2-eth1')
        switch[1].add_link(tgen.gears['r3'], nodeif='r3-eth0')
        switch[1].add_link(tgen.gears['r4'], nodeif='r4-eth0')

        switch[2] = tgen.add_switch('sw2')
        switch[2].add_link(tgen.gears['r2'], nodeif='r2-eth2')
        switch[2].add_link(tgen.gears['r3'], nodeif='r3-eth1')

def ltemplatePreRouterStartHook():
    cc = ltemplateRtrCmd()
    tgen = get_topogen()
    logger.info('pre router-start hook')
    #check for normal init
    if len(tgen.net) == 1:
        logger.info('Topology not configured, skipping setup')
        return False
    return True

def ltemplatePostRouterStartHook():
    logger.info('post router-start hook')
    return True




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 2017 by David Lamparter, placed in public domain

import sys, re, subprocess, os

# find all DEFUNs
defun_re = re.compile(
        r'^((DEF(UN(|_ATTR|_CMD_(ELEMENT|FUNC_(DECL|TEXT))|_DEPRECATED|_NOSH|_HIDDEN|SH(|_ATTR|_DEPRECATED|_HIDDEN))?|PY|PY_ATTR|PY_HIDDEN)|ALIAS)\s*\(.*?)^(?=\s*\{)',
        re.M | re.S)
define_re = re.compile(
        r'((^#\s*define[^\n]+[^\\]\n)+)',
        re.M | re.S)
# find clang-format control that we just inserted
clean_re = re.compile(
        r'^.*/\* \$FRR indent\$ \*/\s*\n\s*/\* clang-format (on|off) \*/\s*\n',
        re.M)

def wrap_file(fn):
    with open(fn, 'r') as fd:
        text = fd.read()

        repl = r'/* $FRR indent$ */\n/* clang-format off */\n' + \
                r'\1' + \
                r'/* $FRR indent$ */\n/* clang-format on */\n'

        # around each DEFUN, insert an indent-on/off comment
        text = defun_re.sub(repl, text)
        text = define_re.sub(repl, text)

        ci = subprocess.Popen(['clang-format'], stdin = subprocess.PIPE, stdout = subprocess.PIPE)
        stdout, ign = ci.communicate(text)
        ci.wait()
        if ci.returncode != 0:
            raise IOError('clang-format returned %d' % (ci.returncode))

        # remove the bits we inserted above
        final = clean_re.sub('', stdout)

        tmpname = fn + '.indent'
        with open(tmpname, 'w') as ofd:
            ofd.write(final)
        os.rename(tmpname, fn)

if __name__ == '__main__':
    for fn in sys.argv[1:]:
        wrap_file(fn)



#
# Modified work Copyright (c) 2019 by VMware, Inc. ("VMware")
# Original work Copyright (c) 2018 by Network Device Education
# Foundation, Inc. ("NetDEF")
#
# Permission to use, copy, modify, and/or distribute this software
# for any purpose with or without fee is hereby granted, provided
# that the above copyright notice and this permission notice appear
# in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND VMWARE DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VMWARE BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
# OF THIS SOFTWARE.
#

from collections import OrderedDict
from json import dumps as json_dumps
from re import search as re_search
import ipaddr
import pytest

# Import topogen and topotest helpers
from lib.topolog import logger

# Required to instantiate the topology builder class.
from lib.common_config import (
    number_to_row, number_to_column,
    load_config_to_router,
    create_interfaces_cfg,
    create_static_routes,
    create_prefix_lists,
    create_route_maps,
    create_bgp_community_lists
)

from lib.bgp import create_router_bgp

ROUTER_LIST = []


def build_topo_from_json(tgen, topo):
    """
    Reads configuration from JSON file. Adds routers, creates interface
    names dynamically and link routers as defined in JSON to create
    topology. Assigns IPs dynamically to all interfaces of each router.

    * `tgen`: Topogen object
    * `topo`: json file data
    """

    ROUTER_LIST = sorted(topo['routers'].keys(),
        key=lambda x: int(re_search('\d+', x).group(0)))

    listRouters = ROUTER_LIST[:]
    for routerN in ROUTER_LIST:
        logger.info('Topo: Add router {}'.format(routerN))
        tgen.add_router(routerN)
        listRouters.append(routerN)

    if 'ipv4base' in topo:
        ipv4Next = ipaddr.IPv4Address(topo['link_ip_start']['ipv4'])
        ipv4Step = 2 ** (32 - topo['link_ip_start']['v4mask'])
        if topo['link_ip_start']['v4mask'] < 32:
            ipv4Next += 1
    if 'ipv6base' in topo:
        ipv6Next = ipaddr.IPv6Address(topo['link_ip_start']['ipv6'])
        ipv6Step = 2 ** (128 - topo['link_ip_start']['v6mask'])
        if topo['link_ip_start']['v6mask'] < 127:
            ipv6Next += 1
    for router in listRouters:
        topo['routers'][router]['nextIfname'] = 0

    while listRouters != []:
        curRouter = listRouters.pop(0)
        # Physical Interfaces
        if 'links' in topo['routers'][curRouter]:
            def link_sort(x):
                if x == 'lo':
                    return 0
                elif 'link' in x:
                    return int(x.split('-link')[1])
                else:
                    return int(re_search('\d+', x).group(0))
            for destRouterLink, data in sorted(topo['routers'][curRouter]['links']. \
                                               iteritems(),
                                               key=lambda x: link_sort(x[0])):
                currRouter_lo_json = \
                    topo['routers'][curRouter]['links'][destRouterLink]
                # Loopback interfaces
                if 'type' in data and data['type'] == 'loopback':
                    if 'ipv4' in currRouter_lo_json and \
                            currRouter_lo_json['ipv4'] == 'auto':
                        currRouter_lo_json['ipv4'] = '{}{}.{}/{}'. \
                            format(topo['lo_prefix']['ipv4'], number_to_row(curRouter), \
                                   number_to_column(curRouter), topo['lo_prefix']['v4mask'])
                    if 'ipv6' in currRouter_lo_json and \
                            currRouter_lo_json['ipv6'] == 'auto':
                        currRouter_lo_json['ipv6'] = '{}{}:{}/{}'. \
                            format(topo['lo_prefix']['ipv6'], number_to_row(curRouter), \
                                   number_to_column(curRouter), topo['lo_prefix']['v6mask'])

                if "-" in destRouterLink:
                    # Spliting and storing destRouterLink data in tempList
                    tempList = destRouterLink.split("-")

                    # destRouter
                    destRouter = tempList.pop(0)

                    # Current Router Link
                    tempList.insert(0, curRouter)
                    curRouterLink = "-".join(tempList)
                else:
                    destRouter = destRouterLink
                    curRouterLink = curRouter

                if destRouter in listRouters:
                    currRouter_link_json = \
                        topo['routers'][curRouter]['links'][destRouterLink]
                    destRouter_link_json = \
                        topo['routers'][destRouter]['links'][curRouterLink]

                    # Assigning name to interfaces
                    currRouter_link_json['interface'] = \
                        '{}-{}-eth{}'.format(curRouter, destRouter, topo['routers'] \
                            [curRouter]['nextIfname'])
                    destRouter_link_json['interface'] = \
                        '{}-{}-eth{}'.format(destRouter, curRouter, topo['routers'] \
                            [destRouter]['nextIfname'])

                    topo['routers'][curRouter]['nextIfname'] += 1
                    topo['routers'][destRouter]['nextIfname'] += 1

                    # Linking routers to each other as defined in JSON file
                    tgen.gears[curRouter].add_link(tgen.gears[destRouter],
                        topo['routers'][curRouter]['links'][destRouterLink] \
                        ['interface'], topo['routers'][destRouter]['links'] \
                        [curRouterLink]['interface'])

                    # IPv4
                    if 'ipv4' in currRouter_link_json:
                        if currRouter_link_json['ipv4'] == 'auto':
                            currRouter_link_json['ipv4'] = \
                                '{}/{}'.format(ipv4Next, topo['link_ip_start'][ \
                                    'v4mask'])
                            destRouter_link_json['ipv4'] = \
                                '{}/{}'.format(ipv4Next + 1, topo['link_ip_start'][ \
                                    'v4mask'])
                            ipv4Next += ipv4Step
                    # IPv6
                    if 'ipv6' in currRouter_link_json:
                        if currRouter_link_json['ipv6'] == 'auto':
                            currRouter_link_json['ipv6'] = \
                                '{}/{}'.format(ipv6Next, topo['link_ip_start'][ \
                                    'v6mask'])
                            destRouter_link_json['ipv6'] = \
                                '{}/{}'.format(ipv6Next + 1, topo['link_ip_start'][ \
                                    'v6mask'])
                            ipv6Next = ipaddr.IPv6Address(int(ipv6Next) + ipv6Step)

            logger.debug("Generated link data for router: %s\n%s", curRouter,
                         json_dumps(topo["routers"][curRouter]["links"],
                                    indent=4, sort_keys=True))


def build_config_from_json(tgen, topo, save_bkup=True):
    """
    Reads initial configuraiton from JSON for each router, builds
    configuration and loads its to router.

    * `tgen`: Topogen object
    * `topo`: json file data
    """

    func_dict = OrderedDict([
        ("links", create_interfaces_cfg),
        ("static_routes", create_static_routes),
        ("prefix_lists", create_prefix_lists),
        ("bgp_community_list", create_bgp_community_lists),
        ("route_maps", create_route_maps),
        ("bgp", create_router_bgp)
    ])

    data = topo["routers"]
    for func_type in func_dict.keys():
        logger.info('Checking for {} configuration in input data'.format(
            func_type))

        func_dict.get(func_type)(tgen, data, build=True)

    for router in sorted(topo['routers'].keys()):
        logger.debug('Configuring router {}...'.format(router))

        result = load_config_to_router(tgen, router, save_bkup)
        if not result:
            logger.info("Failed while configuring {}".format(router))
            pytest.exit(1)




from lutil import luCommand
luCommand('r1','vtysh -c "clear vrf cust1 prefix 99.0.0.1/32"','.','none','Cleared VRF route')
luCommand('r3','vtysh -c "clear vrf cust1 prefix 99.0.0.2/32"','.','none','Cleared VRF route')
luCommand('r4','vtysh -c "clear vrf cust1 prefix 99.0.0.3/32"','.','none','Cleared VRF route')
luCommand('r1','vtysh -c "show vnc registrations local"','99.0.0.1','fail','Local Registration cleared')
luCommand('r3','vtysh -c "show vnc registrations local"','99.0.0.2','fail','Local Registration cleared')
luCommand('r4','vtysh -c "show vnc registrations local"','99.0.0.3','fail','Local Registration cleared')
luCommand('r1','vtysh -c "show bgp ipv4 uni"','2 routes and 2','wait','Unicast SAFI updated')
luCommand('r2','vtysh -c "show bgp ipv4 uni"','No BGP prefixes displayed','pass','Unicast SAFI')
luCommand('r3','vtysh -c "show bgp ipv4 uni"','2 routes and 2','wait','Unicast SAFI updated')
luCommand('r4','vtysh -c "show bgp ipv4 uni"','2 routes and 2','wait','Unicast SAFI updated')
luCommand('ce1','vtysh -c "show bgp ipv4 uni"','2 routes and 2','wait','Local and remote routes')
luCommand('ce2','vtysh -c "show bgp ipv4 uni"','2 routes and 2','wait','Local and remote routes')
luCommand('ce3','vtysh -c "show bgp ipv4 uni"','2 routes and 2','wait','Local and remote routes')
luCommand('r1','vtysh -c "show vnc registrations remote"','Prefix ','fail','Remote Registration cleared')
luCommand('r3','vtysh -c "show vnc registrations remote"','Prefix ','fail','Remote Registration cleared')
luCommand('r4','vtysh -c "show vnc registrations remote"','Prefix ','fail','Remote Registration cleared')



#!/usr/bin/env python

#
# test_bgp_rr_ibgp_topo1.py
#
# Copyright (c) 2019 by
# Cumulus Networks, Inc.
# Donald Sharp
#
# Permission to use, copy, modify, and/or distribute this software
# for any purpose with or without fee is hereby granted, provided
# that the above copyright notice and this permission notice appear
# in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NETDEF DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NETDEF BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
# OF THIS SOFTWARE.
#

"""
test_bgp_rr_ibgp_topo1.py: Testing IBGP with RR and no IGP


    In a leaf/spine topology with only IBGP connections, where
    the same network is being redistributed at multiple points
    in the network ( say a redistribute connected at both leaf and spines )
    we end up in a state where zebra gets very confused.

    eva# show ip route
    Codes: K - kernel route, C - connected, S - static, R - RIP,
           O - OSPF, I - IS-IS, B - BGP, E - EIGRP, N - NHRP,
           T - Table, v - VNC, V - VNC-Direct, A - Babel, D - SHARP,
           F - PBR, f - OpenFabric,
           > - selected route, * - FIB route, q - queued route, r - rejected route
    
    C>* 192.168.1.0/24 is directly connected, tor1-eth0, 00:00:30
    C>* 192.168.2.0/24 is directly connected, tor1-eth1, 00:00:30
    B   192.168.3.0/24 [200/0] via 192.168.4.2 inactive, 00:00:25
                               via 192.168.6.2 inactive, 00:00:25
    B>* 192.168.4.0/24 [200/0] via 192.168.2.3, tor1-eth1, 00:00:25
      *                        via 192.168.6.2 inactive, 00:00:25
    C>* 192.168.5.0/24 is directly connected, tor1-eth2, 00:00:30
    B>* 192.168.6.0/24 [200/0] via 192.168.4.2 inactive, 00:00:25
      *                        via 192.168.5.4, tor1-eth2, 00:00:25

    Effectively we have ibgp routes recursing through ibgp routes
    and there is no metric to discern whom to listen to.
    
    This draft:
    https://tools.ietf.org/html/draft-ietf-idr-bgp-optimal-route-reflection-19
    
    appears to address this issue.  From looking at both cisco and arista
    deployments they are handling this issue by having the route reflector
    prefer the localy learned routes over from their clients.
    
    Add this topology, in a broken state, so that when we do fix this issue
    it is a simple matter of touching this topology up and re-adding it
    to the normal daily builds.  I also wanted to add this topology
    since it is in a state of `doneness` and I wanted to move onto
    my normal day job without having to remember about this test.
    
    This topology is not configured to be run as part of the normal
    topotests.

"""

import os
import re
import sys
import pytest
import json

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, '../'))

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger

# Required to instantiate the topology builder class.
from mininet.topo import Topo

#####################################################
##
##   Network Topology Definition
##
#####################################################

class NetworkTopo(Topo):
    "BGP_RR_IBGP Topology 1"

    def build(self, **_opts):
        "Build function"

        tgen = get_topogen(self)

        tgen.add_router('tor1')
        tgen.add_router('tor2')
        tgen.add_router('spine1')
        tgen.add_router('spine2')

        # First switch is for a dummy interface (for local network)
        # on tor1
	# 192.168.1.0/24
        switch = tgen.add_switch('sw1')
        switch.add_link(tgen.gears['tor1'])

	# 192.168.2.0/24 - tor1 <-> spine1 connection
        switch = tgen.add_switch('sw2')
        switch.add_link(tgen.gears['tor1'])
        switch.add_link(tgen.gears['spine1'])

        # 3rd switch is for a dummy interface (for local netwokr)
	# 192.168.3.0/24 - tor2 
        switch = tgen.add_switch('sw3')
        switch.add_link(tgen.gears['tor2'])

	# 192.168.4.0/24 - tor2 <-> spine1 connection
        switch = tgen.add_switch('sw4')
        switch.add_link(tgen.gears['tor2'])
        switch.add_link(tgen.gears['spine1'])

	# 192.168.5.0/24 - tor1 <-> spine2 connection
        switch = tgen.add_switch('sw5')
        switch.add_link(tgen.gears['tor1'])
        switch.add_link(tgen.gears['spine2'])

	# 192.168.6.0/24 - tor2 <-> spine2 connection
        switch = tgen.add_switch('sw6')
        switch.add_link(tgen.gears['tor2'])
        switch.add_link(tgen.gears['spine2'])

#####################################################
##
##   Tests starting
##
#####################################################

def setup_module(module):
    "Setup topology"
    tgen = Topogen(NetworkTopo, module.__name__)
    tgen.start_topology()

    # This is a sample of configuration loading.
    router_list = tgen.routers()
    for rname, router in router_list.iteritems():
        router.load_config(
            TopoRouter.RD_ZEBRA,
            os.path.join(CWD, '{}/zebra.conf'.format(rname))
        )
        router.load_config(
            TopoRouter.RD_BGP,
            os.path.join(CWD, '{}/bgpd.conf'.format(rname))
        )

    tgen.start_router()
    # tgen.mininet_cli()


def teardown_module(_mod):
    "Teardown the pytest environment"
    tgen = get_topogen()

    # This function tears down the whole topology.
    tgen.stop_topology()


def test_converge_protocols():
    "Wait for protocol convergence"

    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    topotest.sleep(5, 'Waiting for BGP_RR_IBGP convergence')


def test_bgp_rr_ibgp_routes():
    "Test Route Reflection"

    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    # Verify BGP_RR_IBGP Status
    logger.info("Verifying BGP_RR_IBGP routes")

def test_zebra_ipv4_routingTable():
    "Test 'show ip route'"

    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    failures = 0
    router_list = tgen.routers().values()
    for router in router_list:
        output = router.vtysh_cmd('show ip route json', isjson=True)
        refTableFile = '{}/{}/show_ip_route.json_ref'.format(CWD, router.name)
        expected = json.loads(open(refTableFile).read())

        assertmsg = 'Zebra IPv4 Routing Table verification failed for router {}'.format(router.name)
        assert topotest.json_cmp(output, expected) is None, assertmsg

def test_shutdown_check_stderr():
    if os.environ.get('TOPOTESTS_CHECK_STDERR') is None:
        pytest.skip('Skipping test for Stderr output and memory leaks')

    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    logger.info("Verifying unexpected STDERR output from daemons")

    router_list = tgen.routers().values()
    for router in router_list:
        router.stop()

        log = tgen.net[router.name].getStdErr('bgpd')
        if log:
            logger.error('BGPd StdErr Log:' + log)
        log = tgen.net[router.name].getStdErr('zebra')
        if log:
            logger.error('Zebra StdErr Log:' + log)


if __name__ == '__main__':
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))

#
# Auxiliary Functions
#



#!/usr/bin/env python

#
# test_eigrp_topo1.py
#
# Copyright (c) 2017 by
# Cumulus Networks, Inc.
# Donald Sharp
#
# Permission to use, copy, modify, and/or distribute this software
# for any purpose with or without fee is hereby granted, provided
# that the above copyright notice and this permission notice appear
# in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NETDEF DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NETDEF BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
# OF THIS SOFTWARE.
#

"""
test_eigrp_topo1.py: Testing EIGRP

"""

import os
import re
import sys
import pytest
import json

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, '../'))

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger

# Required to instantiate the topology builder class.
from mininet.topo import Topo

#####################################################
##
##   Network Topology Definition
##
#####################################################

class NetworkTopo(Topo):
    "EIGRP Topology 1"

    def build(self, **_opts):
        "Build function"

        tgen = get_topogen(self)

        for routern in range(1, 4):
            tgen.add_router('r{}'.format(routern))

        # On main router
        # First switch is for a dummy interface (for local network)
        switch = tgen.add_switch('sw1')
        switch.add_link(tgen.gears['r1'])

        # Switches for EIGRP
        # switch 2 switch is for connection to EIGRP router
        switch = tgen.add_switch('sw2')
        switch.add_link(tgen.gears['r1'])
        switch.add_link(tgen.gears['r2'])

        # switch 4 is stub on remote EIGRP router
        switch = tgen.add_switch('sw4')
        switch.add_link(tgen.gears['r3'])

        # switch 3 is between EIGRP routers
        switch = tgen.add_switch('sw3')
        switch.add_link(tgen.gears['r2'])
        switch.add_link(tgen.gears['r3'])


#####################################################
##
##   Tests starting
##
#####################################################

def setup_module(module):
    "Setup topology"
    tgen = Topogen(NetworkTopo, module.__name__)
    tgen.start_topology()

    # This is a sample of configuration loading.
    router_list = tgen.routers()
    for rname, router in router_list.iteritems():
        router.load_config(
            TopoRouter.RD_ZEBRA,
            os.path.join(CWD, '{}/zebra.conf'.format(rname))
        )
        router.load_config(
            TopoRouter.RD_EIGRP,
            os.path.join(CWD, '{}/eigrpd.conf'.format(rname))
        )

    tgen.start_router()


def teardown_module(_mod):
    "Teardown the pytest environment"
    tgen = get_topogen()

    # This function tears down the whole topology.
    tgen.stop_topology()


def test_converge_protocols():
    "Wait for protocol convergence"

    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    topotest.sleep(5, 'Waiting for EIGRP convergence')


def test_eigrp_routes():
    "Test EIGRP 'show ip eigrp'"

    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    # Verify EIGRP Status
    logger.info("Verifying EIGRP routes")

    router_list = tgen.routers().values()
    for router in router_list:
        refTableFile = '{}/{}/show_ip_eigrp.json'.format(CWD, router.name)

        # Read expected result from file
        expected = json.loads(open(refTableFile).read())

        # Actual output from router
        actual = ip_eigrp_topo(router)

        assertmsg = '"show ip eigrp topo" mismatches on {}'.format(router.name)
        assert topotest.json_cmp(actual, expected) is None, assertmsg

def test_zebra_ipv4_routingTable():
    "Test 'show ip route'"

    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    failures = 0
    router_list = tgen.routers().values()
    for router in router_list:
        output = router.vtysh_cmd('show ip route json', isjson=True)
        refTableFile = '{}/{}/show_ip_route.json_ref'.format(CWD, router.name)
        expected = json.loads(open(refTableFile).read())

        assertmsg = 'Zebra IPv4 Routing Table verification failed for router {}'.format(router.name)
        assert topotest.json_cmp(output, expected) is None, assertmsg

def test_shut_interface_and_recover():
    "Test shutdown of an interface and recovery of the interface"

    tgen = get_topogen()
    router = tgen.gears['r1']
    router.run('ip link set r1-eth1 down')
    topotest.sleep(5, 'Waiting for EIGRP convergence')
    router.run('ip link set r1-eth1 up')



def test_shutdown_check_stderr():
    if os.environ.get('TOPOTESTS_CHECK_STDERR') is None:
        pytest.skip('Skipping test for Stderr output and memory leaks')

    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    logger.info("Verifying unexpected STDERR output from daemons")

    router_list = tgen.routers().values()
    for router in router_list:
        router.stop()

        log = tgen.net[router.name].getStdErr('eigrpd')
        if log:
            logger.error('EIGRPd StdErr Log:' + log)
        log = tgen.net[router.name].getStdErr('zebra')
        if log:
            logger.error('Zebra StdErr Log:' + log)


if __name__ == '__main__':
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))

#
# Auxiliary Functions
#
def ip_eigrp_topo(node):
    """
    Parse 'show ip eigrp topo' from `node` and returns a dict with the
    result.

    Example:
    {
        'P': {
            '192.168.1.0/24': {
                'sucessors': 1,
                'fd': 112233,
                'serno': 0,
                'via': 'Connected',
                'interface': 'eth0',
            },
            '192.168.2.0/24': {
                'sucessors': 1,
                'fd': 112234,
                'serno': 0,
                'via': 'Connected',
                'interface': 'eth1',
            }
        }
    }
    """
    output = topotest.normalize_text(node.vtysh_cmd('show ip eigrp topo')).splitlines()
    result = {}
    for idx, line in enumerate(output):
        columns = line.split(' ', 1)

        # Parse the following format into python dicts
        # code A.B.C.D/E, X successors, FD is Y, serno: Z
        #       via FOO, interface-name
        code = columns[0]
        if code not in ['P', 'A', 'U', 'Q', 'R', 'r', 's']:
            continue

        if not result.has_key(code):
            result[code] = {}

        # Split network from the rest
        columns = columns[1].split(',')

        # Parse first line data
        network = columns[0]
        result[code][network] = {}
        for column in columns:
            # Skip the network column
            if column == columns[0]:
                continue

            match = re.search(r'(\d+) successors', column)
            if match is not None:
                result[code][network]['successors'] = match.group(1)
                continue

            match = re.search(r'FD is (\d+)', column)
            if match is not None:
                result[code][network]['fd'] = match.group(1)
                continue

            match = re.search(r'serno: (\d+)', column)
            if match is not None:
                result[code][network]['serno'] = match.group(1)
                continue

        # Parse second line data
        nextline = output[idx + 1]
        columns = topotest.normalize_text(nextline).split(',')
        for column in columns:
            match = re.search(r'via (.+)', column)
            if match is not None:
                result[code][network]['via'] = match.group(1)
                continue

            match = re.search(r'(.+)', column)
            if match is not None:
                result[code][network]['interface'] = match.group(1)
                continue

    return result



#!/usr/bin/env python

#
# test_pim.py
#
# Copyright (c) 2018 Cumulus Networks, Inc.
#                    Donald Sharp
#
# Permission to use, copy, modify, and/or distribute this software
# for any purpose with or without fee is hereby granted, provided
# that the above copyright notice and this permission notice appear
# in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND Cumulus Networks DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NETDEF BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
# OF THIS SOFTWARE.
#

"""
test_pim.py: Test pim
"""

import os
import sys
import pytest

CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, '../'))

# pylint: disable=C0413
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger

from mininet.topo import Topo

class PIMTopo(Topo):
    def build(self, *_args, **_opts):
        "Build function"
        tgen = get_topogen(self)

        for routern in range(1, 3):
            tgen.add_router('r{}'.format(routern))

        # r1 <- sw1 -> r2
        sw = tgen.add_switch('sw1')
        sw.add_link(tgen.gears['r1'])
        sw.add_link(tgen.gears['r2'])


def setup_module(mod):
    "Sets up the pytest environment"
    tgen = Topogen(PIMTopo, mod.__name__)
    tgen.start_topology()

    # For all registered routers, load the zebra configuration file
    for rname, router in tgen.routers().iteritems():
        router.load_config(
            TopoRouter.RD_ZEBRA,
            os.path.join(CWD, '{}/zebra.conf'.format(rname))
        )
        router.load_config(
            TopoRouter.RD_PIM,
            os.path.join(CWD, '{}/pimd.conf'.format(rname))
        )

    # After loading the configurations, this function loads configured daemons.
    tgen.start_router()


def teardown_module(mod):
    "Teardown the pytest environment"
    tgen = get_topogen()

    # This function tears down the whole topology.
    tgen.stop_topology()


def test_pim_send_mcast_stream():
    "Establish a Multicast stream from r2 -> r1 and then ensure S,G is created as appropriate"
    logger.info("Establish a Mcast stream from r2->r1 and then ensure S,G created")

    tgen = get_topogen()

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    r2 = tgen.gears['r2']
    r1 = tgen.gears['r1']

    # Let's establish a S,G stream from r2 -> r1
    CWD = os.path.dirname(os.path.realpath(__file__))
    r2.run("{}/mcast-tx.py --ttl 5 --count 5 --interval 10 229.1.1.1 r2-eth0 > /tmp/bar".format(CWD))

    # Let's see that it shows up and we have established some basic state
    out = r1.vtysh_cmd("show ip pim upstream json", isjson=True)
    expected = {
        '229.1.1.1': {
            '10.0.20.2': {
                'firstHopRouter': 1,
                'joinState': 'NotJoined',
                'regState': 'RegPrune',
                'inboundInterface': 'r1-eth0',
            }
        }
    }

    assert topotest.json_cmp(out, expected) is None, 'failed to converge pim'


def test_pim_igmp_report():
    "Send a igmp report from r2->r1 and ensure that the *,G state is created on r1"
    logger.info("Send a igmp report from r2-r1 and ensure *,G created")

    tgen = get_topogen()

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    r2 = tgen.gears['r2']
    r1 = tgen.gears['r1']

    # Let's send a igmp report from r2->r1
    CWD = os.path.dirname(os.path.realpath(__file__))
    r2.run("{}/mcast-rx.py 229.1.1.2 r2-eth0 &".format(CWD))

    out = r1.vtysh_cmd("show ip pim upstream json", isjson=True)
    expected = {
        '229.1.1.2': {
            '*': {
                'sourceIgmp': 1,
                'joinState': 'Joined',
                'regState': 'RegNoInfo',
                'sptBit': 0,
            }
        }
    }

    assert topotest.json_cmp(out, expected) is None, 'failed to converge pim'


def test_memory_leak():
    "Run the memory leak test and report results."
    tgen = get_topogen()
    if not tgen.is_memleak_enabled():
        pytest.skip('Memory leak test/report is disabled')

    tgen.report_memory_leaks()


if __name__ == '__main__':
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python

# Copyright 2018, LabN Consulting, L.L.C.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; see the file COPYING; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#  
# want_rd_routes = [
#     {'rd':'10:1', 'p':'5.1.0.0/24', 'n':'1.1.1.1'},
#     {'rd':'10:1', 'p':'5.1.0.0/24', 'n':'1.1.1.1'},
# 
#     {'rd':'10:3', 'p':'5.1.0.0/24', 'n':'3.3.3.3'},
# ]
# 
# ribRequireVpnRoutes('r2','Customer routes',want_rd_routes)
#
# want_unicast_routes = [
#     {'p':'5.1.0.0/24', 'n':'1.1.1.1'},
# ]
#
# ribRequireUnicastRoutes('r1','ipv4','r1-cust1','Customer routes in vrf',want_unicast_routes)
# ribRequireUnicastRoutes('r1','ipv4','','Customer routes in default',want_unicast_routes)
#

from lutil import luCommand,luResult
import json
import re

# gpz: get rib in json form and compare against desired routes
class BgpRib:
    def routes_include_wanted(self,pfxtbl,want,debug):
	# helper function to RequireVpnRoutes
	for pfx in pfxtbl.iterkeys():
	    if debug:
		print 'trying pfx ' + pfx
	    if pfx != want['p']:
		if debug:
		    print 'want pfx=' + want['p'] + ', not ' + pfx
		continue
	    if debug:
		print 'have pfx=' + pfx
	    for r in pfxtbl[pfx]:
		if debug:
		    print 'trying route'
		nexthops = r['nexthops']
		for nh in nexthops:
		    if debug:
			print 'trying nh ' + nh['ip']
		    if nh['ip'] == want['n']:
			if debug:
			    print 'found ' + want['n']
			return 1
		    else:
			if debug:
			    print 'want nh=' + want['n'] + ', not ' + nh['ip']
	    if debug:
		print 'missing route: pfx=' + want['p'] + ', nh=' + want['n']
	    return 0

    def RequireVpnRoutes(self, target, title, wantroutes, debug=0):
	import json
        logstr = "RequireVpnRoutes " + str(wantroutes)
        #non json form for humans
	luCommand(target,'vtysh -c "show bgp ipv4 vpn"','.','None','Get VPN RIB (non-json)')
	ret = luCommand(target,'vtysh -c "show bgp ipv4 vpn json"','.*','None','Get VPN RIB (json)')
        if re.search(r'^\s*$', ret):
            # degenerate case: empty json means no routes
            if len(wantroutes) > 0:
                luResult(target, False, title, logstr)
                return
            luResult(target, True, title, logstr)
	rib = json.loads(ret)
	rds = rib['routes']['routeDistinguishers']
	for want in wantroutes:
	    found = 0
	    if debug:
		print "want rd " + want['rd']
	    for rd in rds.iterkeys():
		if rd != want['rd']:
		    continue
		if debug:
		    print "found rd " + rd
		table = rds[rd]
		if self.routes_include_wanted(table,want,debug):
		    found = 1
		    break
	    if not found:
		luResult(target, False, title, logstr)
		return
	luResult(target, True, title, logstr)

    def RequireUnicastRoutes(self,target,afi,vrf,title,wantroutes,debug=0):
        logstr = "RequireVpnRoutes " + str(wantroutes)
	vrfstr = ''
	if vrf != '':
	    vrfstr = 'vrf %s' % (vrf)

	if (afi != 'ipv4') and (afi != 'ipv6'):
	    print "ERROR invalid afi";

	cmdstr = 'show bgp %s %s unicast' % (vrfstr, afi)
        #non json form for humans
	cmd = 'vtysh -c "%s"' % cmdstr
	luCommand(target,cmd,'.','None','Get %s %s RIB (non-json)' % (vrfstr, afi))
        cmd = 'vtysh -c "%s json"' % cmdstr
	ret = luCommand(target,cmd,'.*','None','Get %s %s RIB (json)' % (vrfstr, afi))
        if re.search(r'^\s*$', ret):
            # degenerate case: empty json means no routes
            if len(wantroutes) > 0:
                luResult(target, False, title, logstr)
                return
            luResult(target, True, title, logstr)
	rib = json.loads(ret)
        try:
	    table = rib['routes']
        # KeyError: 'routes' probably means missing/bad VRF
        except KeyError as err:
	    if vrf != '':
                errstr = '-script ERROR: check if wrong vrf (%s)' % (vrf)
            else:
                errstr = '-script ERROR: check if vrf missing'
	    luResult(target, False, title + errstr, logstr)
	    return
	for want in wantroutes:
	    if not self.routes_include_wanted(table,want,debug):
		luResult(target, False, title, logstr)
		return
	luResult(target, True, title, logstr)


BgpRib=BgpRib()

def bgpribRequireVpnRoutes(target, title, wantroutes, debug=0):
    BgpRib.RequireVpnRoutes(target, title, wantroutes, debug)

def bgpribRequireUnicastRoutes(target, afi, vrf, title, wantroutes, debug=0):
    BgpRib.RequireUnicastRoutes(target, afi, vrf, title, wantroutes, debug)



from lutil import luCommand
luCommand('r1','vtysh -c "debug rfapi-dev unregister vn 10.0.0.1 un 1.1.1.1 prefix 11.11.11.0/24"','', 'none', 'Prefix removed')
luCommand('r1','vtysh -c "show vnc registrations"','Locally: *Active:  0 ','wait','Local registration removed')
luCommand('r1','vtysh -c "debug rfapi-dev close vn 10.0.0.1 un 1.1.1.1"','status 0', 'pass', 'Closed RFAPI')

luCommand('r3','vtysh -c "debug rfapi-dev unregister vn 10.0.0.2 un 2.2.2.2 prefix 22.22.22.0/24"','', 'none', 'Prefix removed')
luCommand('r3','vtysh -c "show vnc registrations"','Locally: *Active:  0 ','wait','Local registration removed')
luCommand('r3','vtysh -c "debug rfapi-dev close vn 10.0.0.2 un 2.2.2.2"','status 0', 'pass', 'Closed RFAPI')

luCommand('r4','vtysh -c "debug rfapi-dev unregister vn 10.0.0.3 un 3.3.3.3 prefix 33.33.33.0/24"','', 'none', 'Prefix removed')
luCommand('r4','vtysh -c "debug rfapi-dev unregister vn 10.0.0.3 un 3.3.3.3 prefix 11.11.11.0/24"','', 'none', 'MP prefix removed')
luCommand('r4','vtysh -c "show vnc registrations"','Locally: *Active:  0 ','wait','Local registration removed')
#luCommand('r4','vtysh -c "debug rfapi-dev close vn 10.0.0.3 un 3.3.3.3"','status 0', 'pass', 'Closed RFAPI')
luCommand('r4','vtysh -c "clear vnc nve *"','.', 'pass', 'Cleared NVEs')

luCommand('r1','vtysh -c "show vnc registrations"','Locally: *Active:  0 .* Remotely: *Active:  0','wait','All registrations cleared')
luCommand('r3','vtysh -c "show vnc registrations"','Locally: *Active:  0 .* Remotely: *Active:  0','wait','All registrations cleared')
luCommand('r4','vtysh -c "show vnc registrations"','Locally: *Active:  0 .* Remotely: *Active:  0','wait','All registrations cleared')

num = '0 exist'
luCommand('r1','vtysh -c "show bgp ipv4 vpn"',num,'pass','VPN SAFI clear')
luCommand('r2','vtysh -c "show bgp ipv4 vpn"',num,'pass','VPN SAFI clear')
luCommand('r3','vtysh -c "show bgp ipv4 vpn"',num,'pass','VPN SAFI clear')
luCommand('r4','vtysh -c "show bgp ipv4 vpn"',num,'pass','VPN SAFI clear')

luCommand('r1','vtysh -c "show vnc registrations"','Locally: *Active:  0 .* Remotely: *Active:  0 .*In Holddown: *Active:  0','wait','No holddowns',20)
luCommand('r3','vtysh -c "show vnc registrations"','Locally: *Active:  0 .* Remotely: *Active:  0 .*In Holddown: *Active:  0','wait','No holddowns')
luCommand('r4','vtysh -c "show vnc registrations"','Locally: *Active:  0 .* Remotely: *Active:  0 .*In Holddown: *Active:  0','wait','No holddowns')

luCommand('r1','vtysh -c "show vnc summary"','.','none')
luCommand('r3','vtysh -c "show vnc summary"','.','none')
luCommand('r4','vtysh -c "show vnc summary"','.','none')




#!/usr/bin/env python

#
# test_version.py
# Tests for library function: version_cmp().
#
# Copyright (c) 2017 by
# Network Device Education Foundation, Inc. ("NetDEF")
#
# Permission to use, copy, modify, and/or distribute this software
# for any purpose with or without fee is hereby granted, provided
# that the above copyright notice and this permission notice appear
# in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NETDEF DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NETDEF BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
# OF THIS SOFTWARE.
#

"""
Tests for the version_cmp() function.
"""

import os
import sys
import pytest

# Save the Current Working Directory to find lib files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, '../../'))

# pylint: disable=C0413
from lib.topotest import version_cmp

def test_valid_versions():
    "Test valid version compare results"

    curver = '3.0'
    samever = '3'
    oldver = '2.0'
    newver = '3.0.1'
    newerver = '3.0.11'
    vercustom = '3.0-dev'
    verysmallinc = '3.0.0.0.0.0.0.1'

    assert version_cmp(curver, oldver) == 1
    assert version_cmp(curver, newver) == -1
    assert version_cmp(curver, curver) == 0
    assert version_cmp(curver, newerver) == -1
    assert version_cmp(newver, newerver) == -1
    assert version_cmp(curver, samever) == 0
    assert version_cmp(curver, vercustom) == 0
    assert version_cmp(vercustom, vercustom) == 0
    assert version_cmp(vercustom, oldver) == 1
    assert version_cmp(vercustom, newver) == -1
    assert version_cmp(vercustom, samever) == 0
    assert version_cmp(curver, verysmallinc) == -1
    assert version_cmp(newver, verysmallinc) == 1
    assert version_cmp(verysmallinc, verysmallinc) == 0
    assert version_cmp(vercustom, verysmallinc) == -1

def test_invalid_versions():
    "Test invalid version strings"

    curver = '3.0'
    badver1 = '.1'
    badver2 = '-1.0'
    badver3 = '.'
    badver4 = '3.-0.3'

    with pytest.raises(ValueError):
        assert version_cmp(curver, badver1)
        assert version_cmp(curver, badver2)
        assert version_cmp(curver, badver3)
        assert version_cmp(curver, badver4)

def test_regression_1():
    """
    Test regression on the following type of comparison: '3.0.2' > '3'
    Expected result is 1.
    """
    assert version_cmp('3.0.2', '3') == 1



#!/usr/bin/env python

#
# test_bgp_show_ip_bgp_fqdn.py
# Part of NetDEF Topology Tests
#
# Copyright (c) 2019 by
# Donatas Abraitis <donatas.abraitis@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software
# for any purpose with or without fee is hereby granted, provided
# that the above copyright notice and this permission notice appear
# in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NETDEF DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NETDEF BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
# OF THIS SOFTWARE.
#

"""
test_bgp_show_ip_bgp_fqdn.py:
Test if FQND is visible in `show [ip] bgp` output if
`bgp default show-hostname` is toggled.

Topology:
r1 <-- eBGP --> r2 <-- iBGP --> r3

1. Check if both hostname and ip are added to JSON output
for 172.16.255.254/32 on r2.
2. Check if only ip is added to JSON output for 172.16.255.254/32 on r3.
"""

import os
import sys
import json
import time
import pytest
import functools

CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, '../'))

# pylint: disable=C0413
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger
from mininet.topo import Topo

class TemplateTopo(Topo):
    def build(self, *_args, **_opts):
        tgen = get_topogen(self)

        for routern in range(1, 4):
            tgen.add_router("r{}".format(routern))

        switch = tgen.add_switch("s1")
        switch.add_link(tgen.gears["r1"])
        switch.add_link(tgen.gears["r2"])

        switch = tgen.add_switch("s2")
        switch.add_link(tgen.gears["r2"])
        switch.add_link(tgen.gears["r3"])


def setup_module(mod):
    tgen = Topogen(TemplateTopo, mod.__name__)
    tgen.start_topology()

    router_list = tgen.routers()

    for i, (rname, router) in enumerate(router_list.iteritems(), 1):
        router.load_config(
            TopoRouter.RD_ZEBRA,
            os.path.join(CWD, '{}/zebra.conf'.format(rname))
        )
        router.load_config(
            TopoRouter.RD_BGP,
            os.path.join(CWD, '{}/bgpd.conf'.format(rname))
        )

    tgen.start_router()

def teardown_module(mod):
    tgen = get_topogen()
    tgen.stop_topology()

def test_bgp_show_ip_bgp_hostname():
    tgen = get_topogen()

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    def _bgp_converge(router):
        output = json.loads(router.vtysh_cmd("show ip bgp 172.16.255.254/32 json"))
        expected = {"prefix": "172.16.255.254/32"}
        return topotest.json_cmp(output, expected)

    def _bgp_show_nexthop_hostname_and_ip(router):
        output = json.loads(router.vtysh_cmd("show ip bgp json"))
        for nh in output["routes"]["172.16.255.254/32"][0]["nexthops"]:
            if "hostname" in nh and "ip" in nh:
                return True
        return False

    def _bgp_show_nexthop_ip_only(router):
        output = json.loads(router.vtysh_cmd("show ip bgp json"))
        for nh in output["routes"]["172.16.255.254/32"][0]["nexthops"]:
            if "ip" in nh and not "hostname" in nh:
                return True
        return False

    test_func = functools.partial(_bgp_converge, tgen.gears["r2"])
    success, result = topotest.run_and_expect(test_func, None, count=60, wait=0.5)

    test_func = functools.partial(_bgp_converge, tgen.gears["r3"])
    success, result = topotest.run_and_expect(test_func, None, count=60, wait=0.5)

    assert result is None, 'Failed bgp convergence in "{}"'.format(tgen.gears["r2"])
    assert _bgp_show_nexthop_hostname_and_ip(tgen.gears["r2"]) == True

    assert result is None, 'Failed bgp convergence in "{}"'.format(tgen.gears["r3"])
    assert _bgp_show_nexthop_ip_only(tgen.gears["r3"]) == True

if __name__ == '__main__':
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python

#
# Part of NetDEF Topology Tests
#
# Copyright (c) 2018, LabN Consulting, L.L.C.
# Authored by Lou Berger <lberger@labn.net>
#
# Permission to use, copy, modify, and/or distribute this software
# for any purpose with or without fee is hereby granted, provided
# that the above copyright notice and this permission notice appear
# in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NETDEF DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NETDEF BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
# OF THIS SOFTWARE.
#

import os
import sys
import pytest

sys.path.append(os.path.join(os.path.dirname(os.path.realpath(__file__)), '..'))

from lib.ltemplate import *

def test_add_routes():
    CliOnFail = None
    # For debugging, uncomment the next line
    #CliOnFail = 'tgen.mininet_cli'
    CheckFunc = 'ltemplateVersionCheck(\'3.1\')'
    #uncomment next line to start cli *before* script is run
    #CheckFunc = 'ltemplateVersionCheck(\'3.1\', cli=True)'
    ltemplateTest('scripts/add_routes.py', False, CliOnFail, CheckFunc)

def test_adjacencies():
    CliOnFail = None
    # For debugging, uncomment the next line
    #CliOnFail = 'tgen.mininet_cli'
    CheckFunc = 'ltemplateVersionCheck(\'3.1\')'
    #uncomment next line to start cli *before* script is run
    #CheckFunc = 'ltemplateVersionCheck(\'3.1\', cli=True)'
    ltemplateTest('scripts/adjacencies.py', False, CliOnFail, CheckFunc)

def test_check_routes():
    CliOnFail = None
    # For debugging, uncomment the next line
    #CliOnFail = 'tgen.mininet_cli'
    CheckFunc = 'ltemplateVersionCheck(\'3.1\')'
    #uncomment next line to start cli *before* script is run
    #CheckFunc = 'ltemplateVersionCheck(\'3.1\', cli=True)'
    ltemplateTest('scripts/check_routes.py', False, CliOnFail, CheckFunc)

def test_check_close():
    CliOnFail = None
    # For debugging, uncomment the next line
    #CliOnFail = 'tgen.mininet_cli'
    CheckFunc = 'ltemplateVersionCheck(\'3.1\')'
    #uncomment next line to start cli *before* script is run
    #CheckFunc = 'ltemplateVersionCheck(\'3.1\', cli=True)'
    ltemplateTest('scripts/check_close.py', False, CliOnFail, CheckFunc)

def test_check_timeout():
    CliOnFail = None
    # For debugging, uncomment the next line
    #CliOnFail = 'tgen.mininet_cli'
    CheckFunc = 'ltemplateVersionCheck(\'3.1\')'
    #uncomment next line to start cli *before* script is run
    #CheckFunc = 'ltemplateVersionCheck(\'3.1\', cli=True)'
    ltemplateTest('scripts/check_timeout.py', False, CliOnFail, CheckFunc)

def test_cleanup_all():
    CliOnFail = None
    # For debugging, uncomment the next line
    #CliOnFail = 'tgen.mininet_cli'
    CheckFunc = 'ltemplateVersionCheck(\'3.1\')'
    #uncomment next line to start cli *before* script is run
    #CheckFunc = 'ltemplateVersionCheck(\'3.1\', cli=True)'
    ltemplateTest('scripts/cleanup_all.py', False, CliOnFail, CheckFunc)

if __name__ == '__main__':
    retval = pytest.main(["-s"])
    sys.exit(retval)



#!/usr/bin/env python

#
# test_rip_topo1.py
# Part of NetDEF Topology Tests
#
# Copyright (c) 2017 by
# Network Device Education Foundation, Inc. ("NetDEF")
#
# Permission to use, copy, modify, and/or distribute this software
# for any purpose with or without fee is hereby granted, provided
# that the above copyright notice and this permission notice appear
# in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NETDEF DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NETDEF BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
# OF THIS SOFTWARE.
#

"""
test_rip_topo1.py: Testing RIPv2

"""

import os
import re
import sys
import pytest
from time import sleep

from mininet.topo import Topo
from mininet.net import Mininet
from mininet.node import Node, OVSSwitch, Host
from mininet.log import setLogLevel, info
from mininet.cli import CLI
from mininet.link import Intf

from functools import partial

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from lib import topotest

fatal_error = ""


#####################################################
##
##   Network Topology Definition
##
#####################################################

class NetworkTopo(Topo):
    "RIP Topology 1"

    def build(self, **_opts):

        # Setup Routers
        router = {}
        #
        # Setup Main Router
        router[1] = topotest.addRouter(self, 'r1')
        #
        # Setup RIP Routers
        for i in range(2, 4):
            router[i] = topotest.addRouter(self, 'r%s' % i)
        #
        # Setup Switches
        switch = {}
        #
        # On main router
        # First switch is for a dummy interface (for local network)
        switch[1] = self.addSwitch('sw1', cls=topotest.LegacySwitch)
        self.addLink(switch[1], router[1], intfName2='r1-eth0')
        #
        # Switches for RIP
        # switch 2 switch is for connection to RIP router
        switch[2] = self.addSwitch('sw2', cls=topotest.LegacySwitch)
        self.addLink(switch[2], router[1], intfName2='r1-eth1')
        self.addLink(switch[2], router[2], intfName2='r2-eth0')
        # switch 3 is between RIP routers
        switch[3] = self.addSwitch('sw3', cls=topotest.LegacySwitch)
        self.addLink(switch[3], router[2], intfName2='r2-eth1')
        self.addLink(switch[3], router[3], intfName2='r3-eth1')
        # switch 4 is stub on remote RIP router
        switch[4] = self.addSwitch('sw4', cls=topotest.LegacySwitch)
        self.addLink(switch[4], router[3], intfName2='r3-eth0')



#####################################################
##
##   Tests starting
##
#####################################################

def setup_module(module):
    global topo, net

    print("\n\n** %s: Setup Topology" % module.__name__)
    print("******************************************\n")

    print("Cleanup old Mininet runs")
    os.system('sudo mn -c > /dev/null 2>&1')

    thisDir = os.path.dirname(os.path.realpath(__file__))
    topo = NetworkTopo()

    net = Mininet(controller=None, topo=topo)
    net.start()

    # Starting Routers
    #
    for i in range(1, 4):
        net['r%s' % i].loadConf('zebra', '%s/r%s/zebra.conf' % (thisDir, i))
        net['r%s' % i].loadConf('ripd', '%s/r%s/ripd.conf' % (thisDir, i))
        net['r%s' % i].startRouter()

    # For debugging after starting Quagga/FRR daemons, uncomment the next line
    # CLI(net)


def teardown_module(module):
    global net

    print("\n\n** %s: Shutdown Topology" % module.__name__)
    print("******************************************\n")

    # End - Shutdown network
    net.stop()


def test_router_running():
    global fatal_error
    global net

    # Skip if previous fatal error condition is raised
    if (fatal_error != ""):
        pytest.skip(fatal_error)

    print("\n\n** Check if FRR/Quagga is running on each Router node")
    print("******************************************\n")
    sleep(5)

    # Make sure that all daemons are running
    for i in range(1, 4):
        fatal_error = net['r%s' % i].checkRouterRunning()
        assert fatal_error == "", fatal_error

    # For debugging after starting FRR/Quagga daemons, uncomment the next line
    # CLI(net)


def test_converge_protocols():
    global fatal_error
    global net

    # Skip if previous fatal error condition is raised
    if (fatal_error != ""):
        pytest.skip(fatal_error)

    thisDir = os.path.dirname(os.path.realpath(__file__))

    print("\n\n** Waiting for protocols convergence")
    print("******************************************\n")

    # Not really implemented yet - just sleep 60 secs for now
    sleep(60)

    # Make sure that all daemons are still running
    for i in range(1, 4):
        fatal_error = net['r%s' % i].checkRouterRunning()
        assert fatal_error == "", fatal_error

    # For debugging after starting FRR/Quagga daemons, uncomment the next line
    # CLI(net)


def test_rip_status():
    global fatal_error
    global net

    # Skip if previous fatal error condition is raised
    if (fatal_error != ""):
        pytest.skip(fatal_error)

    thisDir = os.path.dirname(os.path.realpath(__file__))

    # Verify RIP Status
    print("\n\n** Verifing RIP status")
    print("******************************************\n")
    failures = 0
    for i in range(1, 4):
        refTableFile = '%s/r%s/rip_status.ref' % (thisDir, i)
        if os.path.isfile(refTableFile):
            # Read expected result from file
            expected = open(refTableFile).read().rstrip()
            # Fix newlines (make them all the same)
            expected = ('\n'.join(expected.splitlines()) + '\n').splitlines(1)

            # Actual output from router
            actual = net['r%s' % i].cmd('vtysh -c "show ip rip status" 2> /dev/null').rstrip()
            # Drop time in next due 
            actual = re.sub(r"in [0-9]+ seconds", "in XX seconds", actual)
            # Drop time in last update
            actual = re.sub(r" [0-2][0-9]:[0-5][0-9]:[0-5][0-9]", " XX:XX:XX", actual)
            # Fix newlines (make them all the same)
            actual = ('\n'.join(actual.splitlines()) + '\n').splitlines(1)

            # Generate Diff
            diff = topotest.get_textdiff(actual, expected,
                title1="actual IP RIP status",
                title2="expected IP RIP status")

            # Empty string if it matches, otherwise diff contains unified diff
            if diff:
                sys.stderr.write('r%s failed IP RIP status check:\n%s\n' % (i, diff))
                failures += 1
            else:
                print("r%s ok" % i)

            assert failures == 0, "IP RIP status failed for router r%s:\n%s" % (i, diff)

    # Make sure that all daemons are still running
    for i in range(1, 4):
        fatal_error = net['r%s' % i].checkRouterRunning()
        assert fatal_error == "", fatal_error

    # For debugging after starting FRR/Quagga daemons, uncomment the next line
    # CLI(net)


def test_rip_routes():
    global fatal_error
    global net

    # Skip if previous fatal error condition is raised
    if (fatal_error != ""):
        pytest.skip(fatal_error)

    thisDir = os.path.dirname(os.path.realpath(__file__))

    # Verify RIP Status
    print("\n\n** Verifing RIP routes")
    print("******************************************\n")
    failures = 0
    for i in range(1, 4):
        refTableFile = '%s/r%s/show_ip_rip.ref' % (thisDir, i)
        if os.path.isfile(refTableFile):
            # Read expected result from file
            expected = open(refTableFile).read().rstrip()
            # Fix newlines (make them all the same)
            expected = ('\n'.join(expected.splitlines()) + '\n').splitlines(1)

            # Actual output from router
            actual = net['r%s' % i].cmd('vtysh -c "show ip rip" 2> /dev/null').rstrip()
            # Drop Time
            actual = re.sub(r"[0-9][0-9]:[0-5][0-9]", "XX:XX", actual)
            # Fix newlines (make them all the same)
            actual = ('\n'.join(actual.splitlines()) + '\n').splitlines(1)

            # Generate Diff
            diff = topotest.get_textdiff(actual, expected,
                title1="actual SHOW IP RIP",
                title2="expected SHOW IP RIP")

            # Empty string if it matches, otherwise diff contains unified diff
            if diff:
                sys.stderr.write('r%s failed SHOW IP RIP check:\n%s\n' % (i, diff))
                failures += 1
            else:
                print("r%s ok" % i)

            assert failures == 0, "SHOW IP RIP failed for router r%s:\n%s" % (i, diff)

    # Make sure that all daemons are still running
    for i in range(1, 4):
        fatal_error = net['r%s' % i].checkRouterRunning()
        assert fatal_error == "", fatal_error

    # For debugging after starting FRR/Quagga daemons, uncomment the next line
    # CLI(net)


def test_zebra_ipv4_routingTable():
    global fatal_error
    global net

    # Skip if previous fatal error condition is raised
    if (fatal_error != ""):
        pytest.skip(fatal_error)

    thisDir = os.path.dirname(os.path.realpath(__file__))

    # Verify OSPFv3 Routing Table
    print("\n\n** Verifing Zebra IPv4 Routing Table")
    print("******************************************\n")
    failures = 0
    for i in range(1, 4):
        refTableFile = '%s/r%s/show_ip_route.ref' % (thisDir, i)
        if os.path.isfile(refTableFile):
            # Read expected result from file
            expected = open(refTableFile).read().rstrip()
            # Fix newlines (make them all the same)
            expected = ('\n'.join(expected.splitlines()) + '\n').splitlines(1)

            # Actual output from router
            actual = net['r%s' % i].cmd('vtysh -c "show ip route" 2> /dev/null | grep "^R"').rstrip()
            # Drop timers on end of line (older Quagga Versions)
            actual = re.sub(r", [0-2][0-9]:[0-5][0-9]:[0-5][0-9]", "", actual)
            # Fix newlines (make them all the same)
            actual = ('\n'.join(actual.splitlines()) + '\n').splitlines(1)

            # Generate Diff
            diff = topotest.get_textdiff(actual, expected,
                title1="actual Zebra IPv4 routing table",
                title2="expected Zebra IPv4 routing table")

            # Empty string if it matches, otherwise diff contains unified diff
            if diff:
                sys.stderr.write('r%s failed Zebra IPv4 Routing Table Check:\n%s\n' % (i, diff))
                failures += 1
            else:
                print("r%s ok" % i)

            assert failures == 0, "Zebra IPv4 Routing Table verification failed for router r%s:\n%s" % (i, diff)

    # Make sure that all daemons are still running
    for i in range(1, 4):
        fatal_error = net['r%s' % i].checkRouterRunning()
        assert fatal_error == "", fatal_error

    # For debugging after starting FRR/Quagga daemons, uncomment the next line
    # CLI(net)


def test_shutdown_check_stderr():
    global fatal_error
    global net

    # Skip if previous fatal error condition is raised
    if (fatal_error != ""):
        pytest.skip(fatal_error)

    if os.environ.get('TOPOTESTS_CHECK_STDERR') is None:
        pytest.skip('Skipping test for Stderr output and memory leaks')

    thisDir = os.path.dirname(os.path.realpath(__file__))

    print("\n\n** Verifing unexpected STDERR output from daemons")
    print("******************************************\n")

    net['r1'].stopRouter()

    log = net['r1'].getStdErr('ripd')
    if log:
        print("\nRIPd StdErr Log:\n" + log)
    log = net['r1'].getStdErr('zebra')
    if log:
        print("\nZebra StdErr Log:\n" + log)


if __name__ == '__main__':

    setLogLevel('info')
    # To suppress tracebacks, either use the following pytest call or add "--tb=no" to cli
    # retval = pytest.main(["-s", "--tb=no"])
    retval = pytest.main(["-s"])
    sys.exit(retval)



from lutil import luCommand
holddownFactorSet = luCommand('r1','vtysh -c "show running"','rfp holddown-factor','none','Holddown factor set')
if not holddownFactorSet:
    to = "-1"
else:
    to = "1"
luCommand('r1','vtysh -c "debug rfapi-dev open vn 20.0.0.1 un 1.1.1.21"','rfapi_set_response_cb: status 0', 'pass', 'Opened RFAPI')
luCommand('r1','vtysh -c "debug rfapi-dev register vn 20.0.0.1 un 1.1.1.21 prefix 111.111.111.0/24 lifetime {}"'.format(to),'', 'none', 'Prefix registered')
luCommand('r1','vtysh -c "show vnc registrations local"','111.111.111.0/24','wait','Local registration',1)
luCommand('r1','vtysh -c "show vnc registrations"','.','none')
luCommand('r3','vtysh -c "show vnc registrations"','111.111.111.0/24','wait','See registration')
luCommand('r4','vtysh -c "show vnc registrations"','111.111.111.0/24','wait','See registration')
luCommand('r1','vtysh -c "debug rfapi-dev close vn 20.0.0.1 un 1.1.1.21"','status 0', 'pass', 'Closed RFAPI')
luCommand('r1','vtysh -c "show vnc registrations"','Locally: *Active:  1 .* Remotely: *Active:  3','wait','See cleanup')
luCommand('r3','vtysh -c "show vnc registrations"','Locally: *Active:  1 .* Remotely: *Active:  3','wait','See cleanup')
luCommand('r4','vtysh -c "show vnc registrations"','Locally: *Active:  2 .* Remotely: *Active:  2','wait','See cleanup')
luCommand('r1','vtysh -c "show vnc registrations"','In Holddown: *Active:  0','wait','Out of holddown',20)
luCommand('r3','vtysh -c "show vnc registrations"','In Holddown: *Active:  0','wait','Out of holddown')
luCommand('r4','vtysh -c "show vnc registrations"','In Holddown: *Active:  0','wait','Out of holddown')



#!/usr/bin/env python

#
# test_bgp_multiview_topo1.py
# Part of NetDEF Topology Tests
#
# Copyright (c) 2016 by
# Network Device Education Foundation, Inc. ("NetDEF")
#
# Permission to use, copy, modify, and/or distribute this software
# for any purpose with or without fee is hereby granted, provided
# that the above copyright notice and this permission notice appear
# in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NETDEF DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NETDEF BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
# OF THIS SOFTWARE.
#

"""
test_bgp_multiview_topo1.py: Simple Quagga/FRR Route-Server Test

+----------+ +----------+ +----------+ +----------+ +----------+
|  peer1   | |  peer2   | |  peer3   | |  peer4   | |  peer5   |
| AS 65001 | | AS 65002 | | AS 65003 | | AS 65004 | | AS 65005 |
+-----+----+ +-----+----+ +-----+----+ +-----+----+ +-----+----+
      | .1         | .2         | .3         | .4         | .5 
      |     ______/            /            /   _________/
       \   /  ________________/            /   /     
        | |  /   _________________________/   /     +----------+  
        | | |  /   __________________________/   ___|  peer6   |
        | | | |  /  ____________________________/.6 | AS 65006 |
        | | | | |  /  _________________________     +----------+
        | | | | | |  /  __________________     \    +----------+ 
        | | | | | | |  /                  \     \___|  peer7   |
        | | | | | | | |                    \     .7 | AS 65007 |
     ~~~~~~~~~~~~~~~~~~~~~                  \       +----------+
   ~~         SW1         ~~                 \      +----------+
   ~~       Switch           ~~               \_____|  peer8   |  
   ~~    172.16.1.0/24     ~~                    .8 | AS 65008 |
     ~~~~~~~~~~~~~~~~~~~~~                          +----------+
              |
              | .254
    +---------+---------+
    |      FRR R1       |
    |   BGP Multi-View  |
    | Peer 1-3 > View 1 |       
    | Peer 4-5 > View 2 |
    | Peer 6-8 > View 3 |
    +---------+---------+
              | .1
              |
        ~~~~~~~~~~~~~        Stub Network is redistributed
      ~~     SW0     ~~      into each BGP view with different
    ~~   172.20.0.1/28  ~~   attributes (using route-map)
      ~~ Stub Switch ~~
        ~~~~~~~~~~~~~
"""  

import os
import re
import sys
import pytest
import glob
from time import sleep

from mininet.topo import Topo
from mininet.net import Mininet
from mininet.node import Node, OVSSwitch, Host
from mininet.log import setLogLevel, info
from mininet.cli import CLI
from mininet.link import Intf

from functools import partial

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from lib import topotest

fatal_error = ""


#####################################################
##
##   Network Topology Definition
##
#####################################################

class NetworkTopo(Topo):
    "BGP Multiview Topology 1"

    def build(self, **_opts):

        exabgpPrivateDirs = ['/etc/exabgp',
                             '/var/run/exabgp',
                             '/var/log']

        # Setup Routers
        router = {}
        for i in range(1, 2):
            router[i] = topotest.addRouter(self, 'r%s' % i)

        # Setup Provider BGP peers
        peer = {}
        for i in range(1, 9):
            peer[i] = self.addHost('peer%s' % i, ip='172.16.1.%s/24' % i,
                                    defaultRoute='via 172.16.1.254',
                                    privateDirs=exabgpPrivateDirs)

        # Setup Switches
        switch = {}
        # First switch is for a dummy interface (for local network)
        switch[0] = self.addSwitch('sw0', cls=topotest.LegacySwitch)
        self.addLink(switch[0], router[1], intfName2='r1-stub')
        # Second switch is for connection to all peering routers
        switch[1] = self.addSwitch('sw1', cls=topotest.LegacySwitch)
        self.addLink(switch[1], router[1], intfName2='r1-eth0')
        for j in range(1, 9):
            self.addLink(switch[1], peer[j], intfName2='peer%s-eth0' % j)


#####################################################
##
##   Tests starting
##
#####################################################

def setup_module(module):
    global topo, net

    print("\n\n** %s: Setup Topology" % module.__name__)
    print("******************************************\n")

    print("Cleanup old Mininet runs")
    os.system('sudo mn -c > /dev/null 2>&1')

    thisDir = os.path.dirname(os.path.realpath(__file__))
    topo = NetworkTopo()

    net = Mininet(controller=None, topo=topo)
    net.start()

    # Starting Routers
    for i in range(1, 2):
        net['r%s' % i].loadConf('zebra', '%s/r%s/zebra.conf' % (thisDir, i))
        net['r%s' % i].loadConf('bgpd', '%s/r%s/bgpd.conf' % (thisDir, i))
        net['r%s' % i].startRouter()

    # Starting PE Hosts and init ExaBGP on each of them
    print('*** Starting BGP on all 8 Peers in 10s')
    sleep(10)
    for i in range(1, 9):
        net['peer%s' % i].cmd('cp %s/exabgp.env /etc/exabgp/exabgp.env' % thisDir)
        net['peer%s' % i].cmd('cp %s/peer%s/* /etc/exabgp/' % (thisDir, i))
        net['peer%s' % i].cmd('chmod 644 /etc/exabgp/*')
        net['peer%s' % i].cmd('chmod 755 /etc/exabgp/*.py')
        net['peer%s' % i].cmd('chown -R exabgp:exabgp /etc/exabgp')
        net['peer%s' % i].cmd('exabgp -e /etc/exabgp/exabgp.env /etc/exabgp/exabgp.cfg')
        print('peer%s' % i),
    print('')

    # For debugging after starting Quagga/FRR daemons, uncomment the next line
    # CLI(net)

def teardown_module(module):
    global net

    print("\n\n** %s: Shutdown Topology" % module.__name__)
    print("******************************************\n")

    # Shutdown - clean up everything
    print('*** Killing BGP on Peer routers')
    # Killing ExaBGP
    for i in range(1, 9):
        net['peer%s' % i].cmd('kill `cat /var/run/exabgp/exabgp.pid`')

    # End - Shutdown network
    net.stop()

def test_router_running():
    global fatal_error
    global net

    # Skip if previous fatal error condition is raised
    if (fatal_error != ""):
        pytest.skip(fatal_error)

    print("\n\n** Check if FRR/Quagga is running on each Router node")
    print("******************************************\n")
    sleep(5)

    # Starting Routers
    for i in range(1, 2):
        fatal_error = net['r%s' % i].checkRouterRunning()
        assert fatal_error == "", fatal_error

    # For debugging after starting FRR/Quagga daemons, uncomment the next line
    # CLI(net)


def test_bgp_converge():
    "Check for BGP converged on all peers and BGP views"

    global fatal_error
    global net

    # Skip if previous fatal error condition is raised
    if (fatal_error != ""):
        pytest.skip(fatal_error)

    # Wait for BGP to converge  (All Neighbors in either Full or TwoWay State)
    print("\n\n** Verify for BGP to converge")
    print("******************************************\n")
    timeout = 60
    while timeout > 0:
        print("Timeout in %s: " % timeout),
        sys.stdout.flush()
        # Look for any node not yet converged
        for i in range(1, 2):
            for view in range(1, 4):
                notConverged = net['r%s' % i].cmd('vtysh -c "show ip bgp view %s summary" 2> /dev/null | grep ^[0-9] | grep -v " 11$"' % view)
                if notConverged:
                    print('Waiting for r%s, view %s' % (i, view))
                    sys.stdout.flush()
                    break
            if notConverged:
                break
        if notConverged:
            sleep(5)
            timeout -= 5
        else:
            print('Done')
            break
    else:
        # Bail out with error if a router fails to converge
        bgpStatus = net['r%s' % i].cmd('vtysh -c "show ip bgp view %s summary"' % view)
        assert False, "BGP did not converge:\n%s" % bgpStatus

    # Wait for an extra 30s to announce all routes
    print('Waiting 30s for routes to be announced');
    sleep(30)
    
    print("BGP converged.")

    # if timeout < 60:
    #     # Only wait if we actually went through a convergence
    #     print("\nwaiting 15s for routes to populate")
    #     sleep(15)

    # Make sure that all daemons are running
    for i in range(1, 2):
        fatal_error = net['r%s' % i].checkRouterRunning()
        assert fatal_error == "", fatal_error

    # For debugging after starting Quagga/FRR daemons, uncomment the next line
    # CLI(net)

def test_bgp_routingTable():
    global fatal_error
    global net

    # Skip if previous fatal error condition is raised
    if (fatal_error != ""):
        pytest.skip(fatal_error)

    thisDir = os.path.dirname(os.path.realpath(__file__))

    print("\n\n** Verifying BGP Routing Tables")
    print("******************************************\n")
    diffresult = {}
    for i in range(1, 2):
        for view in range(1, 4):
            success = 0
            # This glob pattern should work as long as number of views < 10
            for refTableFile in (glob.glob(
                '%s/r%s/show_ip_bgp_view_%s*.ref' % (thisDir, i, view))):

                if os.path.isfile(refTableFile):
                    # Read expected result from file
                    expected = open(refTableFile).read().rstrip()
                    # Fix newlines (make them all the same)
                    expected = ('\n'.join(expected.splitlines()) + '\n').splitlines(1)

                    # Actual output from router
                    actual = net['r%s' % i].cmd('vtysh -c "show ip bgp view %s" 2> /dev/null' % view).rstrip()
        
                    # Fix inconsitent spaces between 0.99.24 and newer versions of Quagga...
                    actual = re.sub('0             0', '0              0', actual)
                    actual = re.sub(r'([0-9])         32768', r'\1          32768', actual)
                    # Remove summary line (changed recently)
                    actual = re.sub(r'Total number.*', '', actual)
                    actual = re.sub(r'Displayed.*', '', actual)
                    actual = actual.rstrip()
                    # Fix table version (ignore it)
                    actual = re.sub(r'(BGP table version is )[0-9]+', r'\1XXX', actual)

                    # Fix newlines (make them all the same)
                    actual = ('\n'.join(actual.splitlines()) + '\n').splitlines(1)

                # Generate Diff
                diff = topotest.get_textdiff(actual, expected,
                    title1="actual BGP routing table",
                    title2="expected BGP routing table")

                if diff:
                    diffresult[refTableFile] = diff
                else:
                    success = 1
                    print("template %s matched: r%s ok" % (refTableFile, i))
                    break;

            if not success:
                resultstr = 'No template matched.\n'
                for f in diffresult.iterkeys():
                    resultstr += (
                        'template %s: r%s failed Routing Table Check for view %s:\n%s\n'
                        % (f, i, view, diffresult[f]))
                raise AssertionError(
                    "Routing Table verification failed for router r%s, view %s:\n%s" % (i, view, resultstr))


    # Make sure that all daemons are running
    for i in range(1, 2):
        fatal_error = net['r%s' % i].checkRouterRunning()
        assert fatal_error == "", fatal_error

    # For debugging after starting FRR/Quagga daemons, uncomment the next line
    # CLI(net)


def test_shutdown_check_stderr():
    global fatal_error
    global net

    # Skip if previous fatal error condition is raised
    if (fatal_error != ""):
        pytest.skip(fatal_error)

    if os.environ.get('TOPOTESTS_CHECK_STDERR') is None:
        print("SKIPPED final check on StdErr output: Disabled (TOPOTESTS_CHECK_STDERR undefined)\n")
        pytest.skip('Skipping test for Stderr output')

    thisDir = os.path.dirname(os.path.realpath(__file__))

    print("\n\n** Verifying unexpected STDERR output from daemons")
    print("******************************************\n")

    net['r1'].stopRouter()

    log = net['r1'].getStdErr('bgpd')
    if log:
        print("\nBGPd StdErr Log:\n" + log)
    log = net['r1'].getStdErr('zebra')
    if log:
        print("\nZebra StdErr Log:\n" + log)


def test_shutdown_check_memleak():
    global fatal_error
    global net

    # Skip if previous fatal error condition is raised
    if (fatal_error != ""):
        pytest.skip(fatal_error)

    if os.environ.get('TOPOTESTS_CHECK_MEMLEAK') is None:
        print("SKIPPED final check on Memory leaks: Disabled (TOPOTESTS_CHECK_MEMLEAK undefined)\n")
        pytest.skip('Skipping test for memory leaks')
    
    thisDir = os.path.dirname(os.path.realpath(__file__))

    net['r1'].stopRouter()
    net['r1'].report_memory_leaks(os.environ.get('TOPOTESTS_CHECK_MEMLEAK'), os.path.basename(__file__))


if __name__ == '__main__':

    setLogLevel('info')
    # To suppress tracebacks, either use the following pytest call or add "--tb=no" to cli
    # retval = pytest.main(["-s", "--tb=no"])
    retval = pytest.main(["-s"])
    sys.exit(retval)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys, os
import subprocess, argparse, tempfile
import indent

def run(cmd):
    proc = subprocess.Popen(cmd, stdout = subprocess.PIPE)
    rv = proc.communicate('')[0].decode('UTF-8')
    proc.wait()
    return rv

clangfmt = run(['git', 'show', 'master:.clang-format'])

argp = argparse.ArgumentParser(description = 'git whitespace-fixing tool')
argp.add_argument('branch', metavar='BRANCH', type = str, nargs = '?', default = 'HEAD')
args = argp.parse_args()

branch = args.branch
commit   = run(['git', 'rev-list', '-n', '1', branch, '--']).strip()

# frr-3.1-dev = first commit that is on master but not on stable/3.0
masterid = run(['git', 'rev-list', '-n', '1', 'frr-3.1-dev', '--']).strip()
masterbase = run(['git', 'merge-base', commit, masterid]).strip()

if masterbase == masterid:
    refbranch = 'master'
else:
    refbranch = '3.0'

sys.stderr.write('autodetected base: %s (can be 3.0 or master)\n' % refbranch)

beforeid = run(['git', 'rev-list', '-n', '1', 'reindent-%s-before' % refbranch, '--']).strip()
afterid  = run(['git', 'rev-list', '-n', '1', 'reindent-%s-after' % refbranch, '--']).strip()

beforebase = run(['git', 'merge-base', commit, beforeid]).strip()
afterbase  = run(['git', 'merge-base', commit, afterid]).strip()

if afterbase == afterid:
    sys.stderr.write('this branch was already rebased\n')
    sys.exit(1)

if beforebase != beforeid:
    sys.stderr.write('you need to rebase your branch onto the tag "reindent-%s-before"\n' % refbranch)
    sys.exit(1)

revs = run(['git', 'rev-list', 'reindent-%s-before..%s' % (refbranch, commit)]).strip().split('\n')
revs.reverse()

srcdir = os.getcwd()
tmpdir = tempfile.mkdtemp('frrindent')
os.chdir(tmpdir)

sys.stderr.write('using temporary directory %s; %d revisions\n' % (tmpdir, len(revs)))
run(['git', 'clone', '-s', '-b', 'reindent-%s-after' % refbranch, srcdir, 'repo'])
os.chdir('repo')

with open('.clang-format', 'w') as fd:
    fd.write(clangfmt)

prev = beforeid
for rev in revs:
    filestat = run(['git', 'diff', '-z', '--name-status', prev, rev]).rstrip('\0').split('\0')
    changes = zip(filestat[0::2], filestat[1::2])
    sys.stderr.write('%s: %d files\n' % (rev, len(changes)))

    for typ, name in changes:
        if typ == 'D':
            run(['git', 'rm', name])
        elif typ in ['A', 'M']:
            run(['git', 'checkout', rev, '--', name])
            if name.endswith('.c') or name.endswith('.h'):
                for d in ['babeld/', 'ldpd/', 'nhrpd/']:
                    if name.startswith(d):
                        break
                else:
                    sys.stderr.write('\t%s\n' % name)
                    indent.wrap_file(name)
            run(['git', 'add', name])

    run(['git', 'commit', '-C', rev])
    prev = rev

run(['git', 'push', 'origin', 'HEAD:refs/heads/reindented-branch'])
sys.stderr.write('\n\n"reindented-branch" should now be OK.\n')
sys.stderr.write('you could use "git reset --hard reindented-branch" to set your current branch to the reindented output\n')
sys.stderr.write('\033[31;1mplease always double-check the output\033[m\n')




#
# Copyright (c) 2019 by VMware, Inc. ("VMware")
# Used Copyright (c) 2018 by Network Device Education Foundation, Inc.
# ("NetDEF") in this file.
#
# Permission to use, copy, modify, and/or distribute this software
# for any purpose with or without fee is hereby granted, provided
# that the above copyright notice and this permission notice appear
# in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND VMWARE DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VMWARE BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
# OF THIS SOFTWARE.
#

from copy import deepcopy
from time import sleep
import traceback
import ipaddr
from lib import topotest

from lib.topolog import logger

# Import common_config to use commomnly used APIs
from lib.common_config import (create_common_configuration,
                               InvalidCLIError,
                               load_config_to_router,
                               check_address_types,
                               generate_ips,
                               find_interface_with_greater_ip,
                               run_frr_cmd, retry)

BGP_CONVERGENCE_TIMEOUT = 10


def create_router_bgp(tgen, topo, input_dict=None, build=False):
    """
    API to configure bgp on router

    Parameters
    ----------
    * `tgen` : Topogen object
    * `topo` : json file data
    * `input_dict` : Input dict data, required when configuring from testcase
    * `build` : Only for initial setup phase this is set as True.

    Usage
    -----
    input_dict = {
        "r1": {
            "bgp": {
                "local_as": "200",
                "router_id": "22.22.22.22",
                "address_family": {
                    "ipv4": {
                        "unicast": {
                            "redistribute": [
                                {"redist_type": "static"},
                                {"redist_type": "connected"}
                            ],
                            "advertise_networks": [
                                {
                                    "network": "20.0.0.0/32",
                                    "no_of_network": 10
                                },
                                {
                                    "network": "30.0.0.0/32",
                                    "no_of_network": 10
                                }
                            ],
                            "neighbor": {
                                "r3": {
                                    "keepalivetimer": 60,
                                    "holddowntimer": 180,
                                    "dest_link": {
                                        "r4": {
                                            "prefix_lists": [
                                                {
                                                    "name": "pf_list_1",
                                                    "direction": "in"
                                                }
                                            ],
                                            "route_maps": [
                                                {"name": "RMAP_MED_R3",
                                                 "direction": "in"}
                                            ],
                                            "next_hop_self": True
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }


    Returns
    -------
    True or False
    """
    logger.debug("Entering lib API: create_router_bgp()")
    result = False
    if not input_dict:
        input_dict = deepcopy(topo)
    else:
        topo = topo["routers"]
        input_dict = deepcopy(input_dict)

    for router in input_dict.keys():
        if "bgp" not in input_dict[router]:
            logger.debug("Router %s: 'bgp' not present in input_dict", router)
            continue

        data_all_bgp = __create_bgp_global(tgen, input_dict, router, build)
        if data_all_bgp:
            bgp_data = input_dict[router]["bgp"]

            bgp_addr_data = bgp_data.setdefault("address_family", {})

            if not bgp_addr_data:
                logger.debug("Router %s: 'address_family' not present in "
                             "input_dict for BGP", router)
            else:

                ipv4_data = bgp_addr_data.setdefault("ipv4", {})
                ipv6_data = bgp_addr_data.setdefault("ipv6", {})

                neigh_unicast = True if ipv4_data.setdefault("unicast", {}) \
                    or ipv6_data.setdefault("unicast", {}) else False

                if neigh_unicast:
                    data_all_bgp = __create_bgp_unicast_neighbor(
                        tgen, topo, input_dict, router,
                        config_data=data_all_bgp)

        try:
            result = create_common_configuration(tgen, router, data_all_bgp,
                                                 "bgp", build)
        except InvalidCLIError:
            # Traceback
            errormsg = traceback.format_exc()
            logger.error(errormsg)
            return errormsg

    logger.debug("Exiting lib API: create_router_bgp()")
    return result


def __create_bgp_global(tgen, input_dict, router, build=False):
    """
    Helper API to create bgp global configuration.

    Parameters
    ----------
    * `tgen` : Topogen object
    * `input_dict` : Input dict data, required when configuring from testcase
    * `router` : router id to be configured.
    * `build` : Only for initial setup phase this is set as True.

    Returns
    -------
    True or False
    """

    logger.debug("Entering lib API: __create_bgp_global()")

    bgp_data = input_dict[router]["bgp"]
    del_bgp_action = bgp_data.setdefault("delete", False)
    if del_bgp_action:
        config_data = ["no router bgp"]

        return config_data

    config_data = []

    if "local_as" not in bgp_data and build:
        logger.error("Router %s: 'local_as' not present in input_dict"
                     "for BGP", router)
        return False

    local_as = bgp_data.setdefault("local_as", "")
    cmd = "router bgp {}".format(local_as)
    vrf_id = bgp_data.setdefault("vrf", None)
    if vrf_id:
        cmd = "{} vrf {}".format(cmd, vrf_id)

    config_data.append(cmd)

    router_id = bgp_data.setdefault("router_id", None)
    del_router_id = bgp_data.setdefault("del_router_id", False)
    if del_router_id:
        config_data.append("no bgp router-id")
    if router_id:
        config_data.append("bgp router-id {}".format(
            router_id))

    return config_data


def __create_bgp_unicast_neighbor(tgen, topo, input_dict, router,
                                  config_data=None):
    """
    Helper API to create configuration for address-family unicast

    Parameters
    ----------
    * `tgen` : Topogen object
    * `topo` : json file data
    * `input_dict` : Input dict data, required when configuring from testcase
    * `router` : router id to be configured.
    * `build` : Only for initial setup phase this is set as True.
    """

    logger.debug("Entering lib API: __create_bgp_unicast_neighbor()")

    add_neigh = True
    if "router bgp" in config_data:
        add_neigh = False
    bgp_data = input_dict[router]["bgp"]["address_family"]

    for addr_type, addr_dict in bgp_data.iteritems():
        if not addr_dict:
            continue

        if not check_address_types(addr_type):
            continue

        addr_data = addr_dict["unicast"]
        if addr_data:
            config_data.append("address-family {} unicast".format(
                addr_type
            ))
        advertise_network = addr_data.setdefault("advertise_networks",
                                                 [])
        for advertise_network_dict in advertise_network:
            network = advertise_network_dict["network"]
            if type(network) is not list:
                network = [network]

            if "no_of_network" in advertise_network_dict:
                no_of_network = advertise_network_dict["no_of_network"]
            else:
                no_of_network = 1

            del_action = advertise_network_dict.setdefault("delete",
                                                           False)

            # Generating IPs for verification
            prefix = str(
                ipaddr.IPNetwork(unicode(network[0])).prefixlen)
            network_list = generate_ips(network, no_of_network)
            for ip in network_list:
                ip = str(ipaddr.IPNetwork(unicode(ip)).network)

                cmd = "network {}/{}".format(ip, prefix)
                if del_action:
                    cmd = "no {}".format(cmd)

                config_data.append(cmd)

        max_paths = addr_data.setdefault("maximum_paths", {})
        if max_paths:
            ibgp = max_paths.setdefault("ibgp", None)
            ebgp = max_paths.setdefault("ebgp", None)
            if ibgp:
                config_data.append("maximum-paths ibgp {}".format(
                    ibgp
                ))
            if ebgp:
                config_data.append("maximum-paths {}".format(
                    ebgp
                ))

        aggregate_addresses = addr_data.setdefault("aggregate_address", [])
        for aggregate_address in aggregate_addresses:
            network = aggregate_address.setdefault("network", None)
            if not network:
                logger.debug("Router %s: 'network' not present in "
                             "input_dict for BGP", router)
            else:
                cmd = "aggregate-address {}".format(network)

                as_set = aggregate_address.setdefault("as_set", False)
                summary = aggregate_address.setdefault("summary", False)
                del_action = aggregate_address.setdefault("delete", False)
                if as_set:
                    cmd = "{} as-set".format(cmd)
                if summary:
                    cmd = "{} summary".format(cmd)

                if del_action:
                    cmd = "no {}".format(cmd)

                config_data.append(cmd)

        redistribute_data = addr_data.setdefault("redistribute", {})
        if redistribute_data:
            for redistribute in redistribute_data:
                if "redist_type" not in redistribute:
                    logger.error("Router %s: 'redist_type' not present in "
                                 "input_dict", router)
                else:
                    cmd = "redistribute {}".format(
                        redistribute["redist_type"])
                    redist_attr = redistribute.setdefault("attribute",
                                                          None)
                    if redist_attr:
                        cmd = "{} {}".format(cmd, redist_attr)
                    del_action = redistribute.setdefault("delete", False)
                    if del_action:
                        cmd = "no {}".format(cmd)
                    config_data.append(cmd)

        if "neighbor" in addr_data:
            neigh_data = __create_bgp_neighbor(topo, input_dict,
                                               router, addr_type, add_neigh)
            config_data.extend(neigh_data)

    for addr_type, addr_dict in bgp_data.iteritems():
        if not addr_dict or not check_address_types(addr_type):
            continue

        addr_data = addr_dict["unicast"]
        if "neighbor" in addr_data:
            neigh_addr_data = __create_bgp_unicast_address_family(
                topo, input_dict, router, addr_type, add_neigh)

            config_data.extend(neigh_addr_data)


    logger.debug("Exiting lib API: __create_bgp_unicast_neighbor()")
    return config_data


def __create_bgp_neighbor(topo, input_dict, router, addr_type, add_neigh=True):
    """
    Helper API to create neighbor specific configuration

    Parameters
    ----------
    * `tgen` : Topogen object
    * `topo` : json file data
    * `input_dict` : Input dict data, required when configuring from testcase
    * `router` : router id to be configured
    """

    config_data = []
    logger.debug("Entering lib API: __create_bgp_neighbor()")

    bgp_data = input_dict[router]["bgp"]["address_family"]
    neigh_data = bgp_data[addr_type]["unicast"]["neighbor"]

    for name, peer_dict in neigh_data.iteritems():
        for dest_link, peer in peer_dict["dest_link"].iteritems():
            nh_details = topo[name]
            remote_as = nh_details["bgp"]["local_as"]
            update_source = None

            if dest_link in nh_details["links"].keys():
                ip_addr = \
                    nh_details["links"][dest_link][addr_type].split("/")[0]
            # Loopback interface
            if "source_link" in peer and peer["source_link"] == "lo":
                update_source = topo[router]["links"]["lo"][
                    addr_type].split("/")[0]

            neigh_cxt = "neighbor {}".format(ip_addr)

            if add_neigh:
                config_data.append("{} remote-as {}".format(neigh_cxt, remote_as))
            if addr_type == "ipv6":
                config_data.append("address-family ipv6 unicast")
                config_data.append("{} activate".format(neigh_cxt))

            disable_connected = peer.setdefault("disable_connected_check",
                                                False)
            keep_alive = peer.setdefault("keep_alive", 60)
            hold_down = peer.setdefault("hold_down", 180)
            password = peer.setdefault("password", None)
            max_hop_limit = peer.setdefault("ebgp_multihop", 1)

            if update_source:
                config_data.append("{} update-source {}".format(
                    neigh_cxt, update_source))
            if disable_connected:
                config_data.append("{} disable-connected-check".format(
                    disable_connected))
            if update_source:
                config_data.append("{} update-source {}".format(neigh_cxt,
                                                                update_source))
            if int(keep_alive) != 60 and int(hold_down) != 180:
                config_data.append(
                    "{} timers {} {}".format(neigh_cxt, keep_alive,
                                             hold_down))
            if password:
                config_data.append(
                    "{} password {}".format(neigh_cxt, password))

            if max_hop_limit > 1:
                config_data.append("{} ebgp-multihop {}".format(neigh_cxt,
                                                                max_hop_limit))
                config_data.append("{} enforce-multihop".format(neigh_cxt))

    logger.debug("Exiting lib API: __create_bgp_unicast_neighbor()")
    return config_data


def __create_bgp_unicast_address_family(topo, input_dict, router, addr_type,
                                        add_neigh=True):
    """
    API prints bgp global config to bgp_json file.

    Parameters
    ----------
    * `bgp_cfg` : BGP class variables have BGP config saved in it for
                  particular router,
    * `local_as_no` : Local as number
    * `router_id` : Router-id
    * `ecmp_path` : ECMP max path
    * `gr_enable` : BGP global gracefull restart config
    """

    config_data = []
    logger.debug("Entering lib API: __create_bgp_unicast_neighbor()")

    bgp_data = input_dict[router]["bgp"]["address_family"]
    neigh_data = bgp_data[addr_type]["unicast"]["neighbor"]

    for peer_name, peer_dict in deepcopy(neigh_data).iteritems():
        for dest_link, peer in peer_dict["dest_link"].iteritems():
            deactivate = None
            nh_details = topo[peer_name]
            # Loopback interface
            if "source_link" in peer and peer["source_link"] == "lo":
                for destRouterLink, data in sorted(nh_details["links"].
                                                   iteritems()):
                    if "type" in data and data["type"] == "loopback":
                        if dest_link == destRouterLink:
                            ip_addr = \
                                nh_details["links"][destRouterLink][
                                    addr_type].split("/")[0]

            # Physical interface
            else:
                if dest_link in nh_details["links"].keys():

                    ip_addr = nh_details["links"][dest_link][
                        addr_type].split("/")[0]
                    if addr_type == "ipv4" and bgp_data["ipv6"]:
                        deactivate = nh_details["links"][
                            dest_link]["ipv6"].split("/")[0]

            neigh_cxt = "neighbor {}".format(ip_addr)
            config_data.append("address-family {} unicast".format(
                addr_type
            ))
            if deactivate:
                config_data.append(
                    "no neighbor {} activate".format(deactivate))

            next_hop_self = peer.setdefault("next_hop_self", None)
            send_community = peer.setdefault("send_community", None)
            prefix_lists = peer.setdefault("prefix_lists", {})
            route_maps = peer.setdefault("route_maps", {})
            no_send_community = peer.setdefault("no_send_community", None)

            # next-hop-self
            if next_hop_self:
                config_data.append("{} next-hop-self".format(neigh_cxt))
            # send_community
            if send_community:
                config_data.append("{} send-community".format(neigh_cxt))

            # no_send_community
            if no_send_community:
                config_data.append("no {} send-community {}".format(
                    neigh_cxt, no_send_community))

            if prefix_lists:
                for prefix_list in prefix_lists:
                    name = prefix_list.setdefault("name", {})
                    direction = prefix_list.setdefault("direction", "in")
                    del_action = prefix_list.setdefault("delete", False)
                    if not name:
                        logger.info("Router %s: 'name' not present in "
                                    "input_dict for BGP neighbor prefix lists",
                                    router)
                    else:
                        cmd = "{} prefix-list {} {}".format(neigh_cxt, name,
                                                            direction)
                        if del_action:
                            cmd = "no {}".format(cmd)
                        config_data.append(cmd)

            if route_maps:
                for route_map in route_maps:
                    name = route_map.setdefault("name", {})
                    direction = route_map.setdefault("direction", "in")
                    del_action = route_map.setdefault("delete", False)
                    if not name:
                        logger.info("Router %s: 'name' not present in "
                                    "input_dict for BGP neighbor route name",
                                    router)
                    else:
                        cmd = "{} route-map {} {}".format(neigh_cxt, name,
                                                          direction)
                        if del_action:
                            cmd = "no {}".format(cmd)
                        config_data.append(cmd)

    return config_data


#############################################
# Verification APIs
#############################################
@retry(attempts=3, wait=2, return_is_str=True)
def verify_router_id(tgen, topo, input_dict):
    """
    Running command "show ip bgp json" for DUT and reading router-id
    from input_dict and verifying with command output.
    1. Statically modfified router-id should take place
    2. When static router-id is deleted highest loopback should
       become router-id
    3. When loopback intf is down then highest physcial intf
       should become router-id

    Parameters
    ----------
    * `tgen`: topogen object
    * `topo`: input json file data
    * `input_dict`: input dictionary, have details of Device Under Test, for
                    which user wants to test the data
    Usage
    -----
    # Verify if router-id for r1 is 12.12.12.12
    input_dict = {
        "r1":{
            "router_id": "12.12.12.12"
        }
    # Verify that router-id for r1 is highest interface ip
    input_dict = {
        "routers": ["r1"]
    }
    result = verify_router_id(tgen, topo, input_dict)

    Returns
    -------
    errormsg(str) or True
    """

    logger.debug("Entering lib API: verify_router_id()")
    for router in input_dict.keys():
        if router not in tgen.routers():
            continue

        rnode = tgen.routers()[router]

        del_router_id = input_dict[router]["bgp"].setdefault(
            "del_router_id", False)

        logger.info("Checking router %s router-id", router)
        show_bgp_json = run_frr_cmd(rnode, "show bgp summary json",
                                        isjson=True)
        router_id_out = show_bgp_json["ipv4Unicast"]["routerId"]
        router_id_out = ipaddr.IPv4Address(unicode(router_id_out))

        # Once router-id is deleted, highest interface ip should become
        # router-id
        if del_router_id:
            router_id = find_interface_with_greater_ip(topo, router)
        else:
            router_id = input_dict[router]["bgp"]["router_id"]
        router_id = ipaddr.IPv4Address(unicode(router_id))

        if router_id == router_id_out:
            logger.info("Found expected router-id %s for router %s",
                        router_id, router)
        else:
            errormsg = "Router-id for router:{} mismatch, expected:" \
                       " {} but found:{}".format(router, router_id,
                                                 router_id_out)
            return errormsg

    logger.debug("Exiting lib API: verify_router_id()")
    return True


@retry(attempts=20, wait=2, return_is_str=True)
def verify_bgp_convergence(tgen, topo):
    """
    API will verify if BGP is converged with in the given time frame.
    Running "show bgp summary json" command and verify bgp neighbor
    state is established,
    Parameters
    ----------
    * `tgen`: topogen object
    * `topo`: input json file data
    * `addr_type`: ip_type, ipv4/ipv6
    Usage
    -----
    # To veriry is BGP is converged for all the routers used in
    topology
    results = verify_bgp_convergence(tgen, topo, "ipv4")
    Returns
    -------
    errormsg(str) or True
    """

    logger.debug("Entering lib API: verify_bgp_convergence()")
    for router, rnode in tgen.routers().iteritems():
        logger.info("Verifying BGP Convergence on router %s", router)
        show_bgp_json = run_frr_cmd(rnode, "show bgp summary json",
                                    isjson=True)
        # Verifying output dictionary show_bgp_json is empty or not
        if not bool(show_bgp_json):
            errormsg = "BGP is not running"
            return errormsg

        # To find neighbor ip type
        bgp_addr_type = topo["routers"][router]["bgp"]["address_family"]
        for addr_type in bgp_addr_type.keys():
            if not check_address_types(addr_type):
                continue
            total_peer = 0

            bgp_neighbors = bgp_addr_type[addr_type]["unicast"]["neighbor"]

            for bgp_neighbor in bgp_neighbors:
                total_peer += len(bgp_neighbors[bgp_neighbor]["dest_link"])

        for addr_type in bgp_addr_type.keys():
            if not check_address_types(addr_type):
                continue
            bgp_neighbors = bgp_addr_type[addr_type]["unicast"]["neighbor"]

            no_of_peer = 0
            for bgp_neighbor, peer_data in bgp_neighbors.iteritems():
                for dest_link in peer_data["dest_link"].keys():
                    data = topo["routers"][bgp_neighbor]["links"]
                    if dest_link in data:
                        neighbor_ip = \
                            data[dest_link][addr_type].split("/")[0]
                        if addr_type == "ipv4":
                            ipv4_data = show_bgp_json["ipv4Unicast"][
                                "peers"]
                            nh_state = ipv4_data[neighbor_ip]["state"]
                        else:
                            ipv6_data = show_bgp_json["ipv6Unicast"][
                                "peers"]
                            nh_state = ipv6_data[neighbor_ip]["state"]

                        if nh_state == "Established":
                            no_of_peer += 1
        if no_of_peer == total_peer:
            logger.info("BGP is Converged for router %s", router)
        else:
            errormsg = "BGP is not converged for router {}".format(
                router)
            return errormsg

    logger.debug("Exiting API: verify_bgp_convergence()")
    return True


def modify_as_number(tgen, topo, input_dict):
    """
    API reads local_as and remote_as from user defined input_dict and
    modify router"s ASNs accordingly. Router"s config is modified and
    recent/changed config is loadeded to router.

    Parameters
    ----------
    * `tgen`  : Topogen object
    * `topo`  : json file data
    * `input_dict` :  defines for which router ASNs needs to be modified

    Usage
    -----
    To modify ASNs for router r1
    input_dict = {
        "r1": {
            "bgp": {
                "local_as": 131079
            }
        }
    result = modify_as_number(tgen, topo, input_dict)

    Returns
    -------
    errormsg(str) or True
    """

    logger.debug("Entering lib API: modify_as_number()")
    try:

        new_topo = deepcopy(topo["routers"])
        router_dict = {}
        for router in input_dict.keys():
            # Remove bgp configuration

            router_dict.update({
                router: {
                    "bgp": {
                        "delete": True
                    }
                }
            })

            new_topo[router]["bgp"]["local_as"] = \
                input_dict[router]["bgp"]["local_as"]

        logger.info("Removing bgp configuration")
        create_router_bgp(tgen, topo, router_dict)

        logger.info("Applying modified bgp configuration")
        create_router_bgp(tgen, new_topo)

    except Exception as e:
        # handle any exception
        logger.error("Error %s occured. Arguments %s.", e.message, e.args)

        # Traceback
        errormsg = traceback.format_exc()
        logger.error(errormsg)
        return errormsg

    logger.debug("Exiting lib API: modify_as_number()")

    return True


@retry(attempts=3, wait=2, return_is_str=True)
def verify_as_numbers(tgen, topo, input_dict):
    """
    This API is to verify AS numbers for given DUT by running
    "show ip bgp neighbor json" command. Local AS and Remote AS
    will ve verified with input_dict data and command output.

    Parameters
    ----------
    * `tgen`: topogen object
    * `topo`: input json file data
    * `addr_type` : ip type, ipv4/ipv6
    * `input_dict`: defines - for which router, AS numbers needs to be verified

    Usage
    -----
    input_dict = {
        "r1": {
            "bgp": {
                "local_as": 131079
            }
        }
    }
    result = verify_as_numbers(tgen, topo, addr_type, input_dict)

    Returns
    -------
    errormsg(str) or True
    """

    logger.debug("Entering lib API: verify_as_numbers()")
    for router in input_dict.keys():
        if router not in tgen.routers():
            continue

        rnode = tgen.routers()[router]

        logger.info("Verifying AS numbers for  dut %s:", router)

        show_ip_bgp_neighbor_json = run_frr_cmd(rnode,
            "show ip bgp neighbor json", isjson=True)
        local_as = input_dict[router]["bgp"]["local_as"]
        bgp_addr_type = topo["routers"][router]["bgp"]["address_family"]

        for addr_type in bgp_addr_type:
            if not check_address_types(addr_type):
                continue

            bgp_neighbors = bgp_addr_type[addr_type]["unicast"][
                "neighbor"]

            for bgp_neighbor, peer_data in bgp_neighbors.iteritems():
                remote_as = input_dict[bgp_neighbor]["bgp"]["local_as"]
                for dest_link, peer_dict in peer_data["dest_link"].iteritems():
                    neighbor_ip = None
                    data = topo["routers"][bgp_neighbor]["links"]

                    if dest_link in data:
                        neighbor_ip = data[dest_link][addr_type]. \
                            split("/")[0]
                    neigh_data = show_ip_bgp_neighbor_json[neighbor_ip]
                    # Verify Local AS for router
                    if neigh_data["localAs"] != local_as:
                        errormsg = "Failed: Verify local_as for dut {}," \
                                   " found: {} but expected: {}".format(
                                       router, neigh_data["localAs"],
                                       local_as)
                        return errormsg
                    else:
                        logger.info("Verified local_as for dut %s, found"
                                    " expected: %s", router, local_as)

                    # Verify Remote AS for neighbor
                    if neigh_data["remoteAs"] != remote_as:
                        errormsg = "Failed: Verify remote_as for dut " \
                                   "{}'s neighbor {}, found: {} but " \
                                   "expected: {}".format(
                                       router, bgp_neighbor,
                                       neigh_data["remoteAs"], remote_as)
                        return errormsg
                    else:
                        logger.info("Verified remote_as for dut %s's "
                                    "neighbor %s, found expected: %s",
                                    router, bgp_neighbor, remote_as)

    logger.debug("Exiting lib API: verify_AS_numbers()")
    return True


def clear_bgp_and_verify(tgen, topo, router):
    """
    This API is to clear bgp neighborship and verify bgp neighborship
    is coming up(BGP is converged) usinf "show bgp summary json" command
    and also verifying for all bgp neighbors uptime before and after
    clear bgp sessions is different as the uptime must be changed once
    bgp sessions are cleared using "clear ip bgp */clear bgp ipv6 *" cmd.

    Parameters
    ----------
    * `tgen`: topogen object
    * `topo`: input json file data
    * `router`: device under test

    Usage
    -----
    result = clear_bgp_and_verify(tgen, topo, addr_type, dut)

    Returns
    -------
    errormsg(str) or True
    """

    logger.debug("Entering lib API: clear_bgp_and_verify()")

    if router not in tgen.routers():
        return False

    rnode = tgen.routers()[router]

    peer_uptime_before_clear_bgp = {}
    # Verifying BGP convergence before bgp clear command
    for retry in range(31):
        sleeptime = 3
        # Waiting for BGP to converge
        logger.info("Waiting for %s sec for BGP to converge on router"
                    " %s...", sleeptime, router)
        sleep(sleeptime)

        show_bgp_json = run_frr_cmd(rnode, "show bgp summary json",
                                        isjson=True)
        # Verifying output dictionary show_bgp_json is empty or not
        if not bool(show_bgp_json):
            errormsg = "BGP is not running"
            return errormsg

        # To find neighbor ip type
        bgp_addr_type = topo["routers"][router]["bgp"]["address_family"]
        total_peer = 0
        for addr_type in bgp_addr_type.keys():

            if not check_address_types(addr_type):
                continue

            bgp_neighbors = bgp_addr_type[addr_type]["unicast"]["neighbor"]

            for bgp_neighbor in bgp_neighbors:
                total_peer += len(bgp_neighbors[bgp_neighbor]["dest_link"])

        no_of_peer = 0
        for addr_type in bgp_addr_type:
            bgp_neighbors = bgp_addr_type[addr_type]["unicast"]["neighbor"]

            for bgp_neighbor, peer_data in bgp_neighbors.iteritems():
                for dest_link, peer_dict in peer_data["dest_link"].iteritems():
                    data = topo["routers"][bgp_neighbor]["links"]

                    if dest_link in data:
                        neighbor_ip = data[dest_link][addr_type].split("/")[0]
                        if addr_type == "ipv4":
                            ipv4_data = show_bgp_json["ipv4Unicast"][
                                "peers"]
                            nh_state = ipv4_data[neighbor_ip]["state"]

                            # Peer up time dictionary
                            peer_uptime_before_clear_bgp[bgp_neighbor] = \
                                ipv4_data[neighbor_ip]["peerUptimeEstablishedEpoch"]
                        else:
                            ipv6_data = show_bgp_json["ipv6Unicast"][
                                "peers"]
                            nh_state = ipv6_data[neighbor_ip]["state"]

                            # Peer up time dictionary
                            peer_uptime_before_clear_bgp[bgp_neighbor] = \
                                ipv6_data[neighbor_ip]["peerUptimeEstablishedEpoch"]

                        if nh_state == "Established":
                            no_of_peer += 1

        if no_of_peer == total_peer:
            logger.info("BGP is Converged for router %s before bgp"
                        " clear", router)
            break
        else:
            logger.info("BGP is not yet Converged for router %s "
                        "before bgp clear", router)
    else:
        errormsg = "TIMEOUT!! BGP is not converged in 30 seconds for" \
                   " router {}".format(router)
        return errormsg

    logger.info(peer_uptime_before_clear_bgp)
    # Clearing BGP
    logger.info("Clearing BGP neighborship for router %s..", router)
    for addr_type in bgp_addr_type.keys():
        if addr_type == "ipv4":
            run_frr_cmd(rnode, "clear ip bgp *")
        elif addr_type == "ipv6":
            run_frr_cmd(rnode, "clear bgp ipv6 *")

    peer_uptime_after_clear_bgp = {}
    # Verifying BGP convergence after bgp clear command
    for retry in range(31):
        sleeptime = 3
        # Waiting for BGP to converge
        logger.info("Waiting for %s sec for BGP to converge on router"
                    " %s...", sleeptime, router)
        sleep(sleeptime)


        show_bgp_json = run_frr_cmd(rnode, "show bgp summary json",
                                        isjson=True)
        # Verifying output dictionary show_bgp_json is empty or not
        if not bool(show_bgp_json):
            errormsg = "BGP is not running"
            return errormsg

        # To find neighbor ip type
        bgp_addr_type = topo["routers"][router]["bgp"]["address_family"]
        total_peer = 0
        for addr_type in bgp_addr_type.keys():
            if not check_address_types(addr_type):
                continue

            bgp_neighbors = bgp_addr_type[addr_type]["unicast"]["neighbor"]

            for bgp_neighbor in bgp_neighbors:
                total_peer += len(bgp_neighbors[bgp_neighbor]["dest_link"])

        no_of_peer = 0
        for addr_type in bgp_addr_type:
            bgp_neighbors = bgp_addr_type[addr_type]["unicast"]["neighbor"]

            for bgp_neighbor, peer_data in bgp_neighbors.iteritems():
                for dest_link, peer_dict in peer_data["dest_link"].iteritems():
                    data = topo["routers"][bgp_neighbor]["links"]

                    if dest_link in data:
                        neighbor_ip = data[dest_link][addr_type].\
                            split("/")[0]
                        if addr_type == "ipv4":
                            ipv4_data = show_bgp_json["ipv4Unicast"][
                                "peers"]
                            nh_state = ipv4_data[neighbor_ip]["state"]
                            peer_uptime_after_clear_bgp[bgp_neighbor] = \
                                ipv4_data[neighbor_ip]["peerUptimeEstablishedEpoch"]
                        else:
                            ipv6_data = show_bgp_json["ipv6Unicast"][
                                "peers"]
                            nh_state = ipv6_data[neighbor_ip]["state"]
                            # Peer up time dictionary
                            peer_uptime_after_clear_bgp[bgp_neighbor] = \
                                ipv6_data[neighbor_ip]["peerUptimeEstablishedEpoch"]

                        if nh_state == "Established":
                            no_of_peer += 1

        if no_of_peer == total_peer:
            logger.info("BGP is Converged for router %s after bgp clear",
                        router)
            break
        else:
            logger.info("BGP is not yet Converged for router %s after"
                        " bgp clear", router)
    else:
        errormsg = "TIMEOUT!! BGP is not converged in 30 seconds for" \
                   " router {}".format(router)
        return errormsg
    logger.info(peer_uptime_after_clear_bgp)
    # Comparing peerUptimeEstablishedEpoch dictionaries
    if peer_uptime_before_clear_bgp != peer_uptime_after_clear_bgp:
        logger.info("BGP neighborship is reset after clear BGP on router %s",
                    router)
    else:
        errormsg = "BGP neighborship is not reset after clear bgp on router" \
                   " {}".format(router)
        return errormsg

    logger.debug("Exiting lib API: clear_bgp_and_verify()")
    return True


def verify_bgp_timers_and_functionality(tgen, topo, input_dict):
    """
    To verify BGP timer config, execute "show ip bgp neighbor json" command
    and verify bgp timers with input_dict data.
    To veirfy bgp timers functonality, shutting down peer interface
    and verify BGP neighborship status.

    Parameters
    ----------
    * `tgen`: topogen object
    * `topo`: input json file data
    * `addr_type`: ip type, ipv4/ipv6
    * `input_dict`: defines for which router, bgp timers needs to be verified

    Usage:
    # To verify BGP timers for neighbor r2 of router r1
    input_dict = {
        "r1": {
           "bgp": {
               "bgp_neighbors":{
                  "r2":{
                      "keepalivetimer": 5,
                      "holddowntimer": 15,
                   }}}}}
    result = verify_bgp_timers_and_functionality(tgen, topo, "ipv4",
        input_dict)

    Returns
    -------
    errormsg(str) or True
    """

    logger.debug("Entering lib API: verify_bgp_timers_and_functionality()")
    sleep(5)
    router_list = tgen.routers()
    for router in input_dict.keys():
        if router not in router_list:
            continue

        rnode = router_list[router]

        logger.info("Verifying bgp timers functionality, DUT is %s:",
                    router)

        show_ip_bgp_neighbor_json = \
            run_frr_cmd(rnode, "show ip bgp neighbor json", isjson=True)

        bgp_addr_type = input_dict[router]["bgp"]["address_family"]

        for addr_type in bgp_addr_type:
            if not check_address_types(addr_type):
                continue

            bgp_neighbors = bgp_addr_type[addr_type]["unicast"][
                "neighbor"]
            for bgp_neighbor, peer_data in bgp_neighbors.iteritems():
                for dest_link, peer_dict in peer_data["dest_link"].iteritems():
                    data = topo["routers"][bgp_neighbor]["links"]

                    keepalivetimer = peer_dict["keepalivetimer"]
                    holddowntimer = peer_dict["holddowntimer"]

                    if dest_link in data:
                        neighbor_ip = data[dest_link][addr_type]. \
                            split("/")[0]
                        neighbor_intf = data[dest_link]["interface"]

                    # Verify HoldDownTimer for neighbor
                    bgpHoldTimeMsecs = show_ip_bgp_neighbor_json[
                        neighbor_ip]["bgpTimerHoldTimeMsecs"]
                    if bgpHoldTimeMsecs != holddowntimer * 1000:
                        errormsg = "Verifying holddowntimer for bgp " \
                                   "neighbor {} under dut {}, found: {} " \
                                   "but expected: {}".format(
                            neighbor_ip, router,
                            bgpHoldTimeMsecs,
                            holddowntimer * 1000)
                        return errormsg

                    # Verify KeepAliveTimer for neighbor
                    bgpKeepAliveTimeMsecs = show_ip_bgp_neighbor_json[
                        neighbor_ip]["bgpTimerKeepAliveIntervalMsecs"]
                    if bgpKeepAliveTimeMsecs != keepalivetimer * 1000:
                        errormsg = "Verifying keepalivetimer for bgp " \
                                   "neighbor {} under dut {}, found: {} " \
                                   "but expected: {}".format(
                            neighbor_ip, router,
                            bgpKeepAliveTimeMsecs,
                            keepalivetimer * 1000)
                        return errormsg

                    ####################
                    # Shutting down peer interface after keepalive time and
                    # after some time bringing up peer interface.
                    # verifying BGP neighborship in (hold down-keep alive)
                    # time, it should not go down
                    ####################

                    # Wait till keep alive time
                    logger.info("=" * 20)
                    logger.info("Scenario 1:")
                    logger.info("Shutdown and bring up peer interface: %s "
                                "in keep alive time : %s sec and verify "
                                " BGP neighborship  is intact in %s sec ",
                                neighbor_intf, keepalivetimer,
                                (holddowntimer - keepalivetimer))
                    logger.info("=" * 20)
                    logger.info("Waiting for %s sec..", keepalivetimer)
                    sleep(keepalivetimer)

                    # Shutting down peer ineterface
                    logger.info("Shutting down interface %s on router %s",
                                neighbor_intf, bgp_neighbor)
                    topotest.interface_set_status(
                        router_list[bgp_neighbor], neighbor_intf,
                        ifaceaction=False)

                    # Bringing up peer interface
                    sleep(5)
                    logger.info("Bringing up interface %s on router %s..",
                                neighbor_intf, bgp_neighbor)
                    topotest.interface_set_status(
                        router_list[bgp_neighbor], neighbor_intf,
                        ifaceaction=True)

                # Verifying BGP neighborship is intact in
                # (holddown - keepalive) time
                for timer in range(keepalivetimer, holddowntimer,
                                   int(holddowntimer / 3)):
                    logger.info("Waiting for %s sec..", keepalivetimer)
                    sleep(keepalivetimer)
                    sleep(2)
                    show_bgp_json = \
                        run_frr_cmd(rnode, "show bgp summary json",
                                        isjson=True)

                    if addr_type == "ipv4":
                        ipv4_data = show_bgp_json["ipv4Unicast"]["peers"]
                        nh_state = ipv4_data[neighbor_ip]["state"]
                    else:
                        ipv6_data = show_bgp_json["ipv6Unicast"]["peers"]
                        nh_state = ipv6_data[neighbor_ip]["state"]

                    if timer == \
                            (holddowntimer - keepalivetimer):
                        if nh_state != "Established":
                            errormsg = "BGP neighborship has not  gone " \
                                       "down in {} sec for neighbor {}" \
                                .format(timer, bgp_neighbor)
                            return errormsg
                        else:
                            logger.info("BGP neighborship is intact in %s"
                                        " sec for neighbor %s",
                                        timer, bgp_neighbor)

                ####################
                # Shutting down peer interface and verifying that BGP
                # neighborship is going down in holddown time
                ####################
                logger.info("=" * 20)
                logger.info("Scenario 2:")
                logger.info("Shutdown peer interface: %s and verify BGP"
                            " neighborship has gone down in hold down "
                            "time %s sec", neighbor_intf, holddowntimer)
                logger.info("=" * 20)

                logger.info("Shutting down interface %s on router %s..",
                            neighbor_intf, bgp_neighbor)
                topotest.interface_set_status(router_list[bgp_neighbor],
                                              neighbor_intf,
                                              ifaceaction=False)

                # Verifying BGP neighborship is going down in holddown time
                for timer in range(keepalivetimer,
                                   (holddowntimer + keepalivetimer),
                                   int(holddowntimer / 3)):
                    logger.info("Waiting for %s sec..", keepalivetimer)
                    sleep(keepalivetimer)
                    sleep(2)
                    show_bgp_json = \
                        run_frr_cmd(rnode, "show bgp summary json",
                                        isjson=True)

                    if addr_type == "ipv4":
                        ipv4_data = show_bgp_json["ipv4Unicast"]["peers"]
                        nh_state = ipv4_data[neighbor_ip]["state"]
                    else:
                        ipv6_data = show_bgp_json["ipv6Unicast"]["peers"]
                        nh_state = ipv6_data[neighbor_ip]["state"]

                    if timer == holddowntimer:
                        if nh_state == "Established":
                            errormsg = "BGP neighborship has not gone " \
                                       "down in {} sec for neighbor {}" \
                                .format(timer, bgp_neighbor)
                            return errormsg
                        else:
                            logger.info("BGP neighborship has gone down in"
                                        " %s sec for neighbor %s",
                                        timer, bgp_neighbor)

    logger.debug("Exiting lib API: verify_bgp_timers_and_functionality()")
    return True


@retry(attempts=3, wait=4, return_is_str=True)
def verify_bgp_attributes(tgen, addr_type, dut, static_routes, rmap_name,
                          input_dict, seq_id=None):
    """
    API will verify BGP attributes set by Route-map for given prefix and
    DUT. it will run "show bgp ipv4/ipv6 {prefix_address} json" command
    in DUT to verify BGP attributes set by route-map, Set attributes
    values will be read from input_dict and verified with command output.

    * `tgen`: topogen object
    * `addr_type` : ip type, ipv4/ipv6
    * `dut`: Device Under Test
    * `static_routes`: Static Routes for which BGP set attributes needs to be
                       verified
    * `rmap_name`: route map name for which set criteria needs to be verified
    * `input_dict`: defines for which router, AS numbers needs
    * `seq_id`: sequence number of rmap, default is None

    Usage
    -----
    input_dict = {
        "r3": {
            "route_maps": {
                "rmap_match_pf_1_ipv4": [{
                    "action": "permit",
                    'seq_id': '5',
                    "match": {
                        addr_type: {
                            "prefix_lists": "pf_list_1_" + addr_type
                        }
                    },
                    "set": {
                        "localpref": 150,
                        "weight": 100
                    }
                }],
                "rmap_match_pf_2_ipv6": [{
                    "action": "permit",
                    'seq_id': '5',
                    "match": {
                        addr_type: {
                            "prefix_lists": "pf_list_1_" + addr_type
                        }
                    },
                    "set": {
                        "med": 50
                    }
                }]
            }
        }
    }
    result = verify_bgp_attributes(tgen, 'ipv4', "r1", "10.0.20.1/32",
                                   rmap_match_pf_1_ipv4, input_dict)

    Returns
    -------
    errormsg(str) or True
    """

    logger.debug("Entering lib API: verify_bgp_attributes()")
    for router, rnode in tgen.routers().iteritems():
        if router != dut:
            continue

        logger.info('Verifying BGP set attributes for dut {}:'.format(router))

        for static_route in static_routes:
            cmd = "show bgp {} {} json".format(addr_type, static_route)
            show_bgp_json = run_frr_cmd(rnode, cmd, isjson=True)
            print("show_bgp_json $$$$$", show_bgp_json)

            dict_to_test = []
            tmp_list = []
            for rmap_router in input_dict.keys():
                for rmap, values in input_dict[rmap_router][
                        "route_maps"].items():
                    print("rmap == rmap_name $$$$1", rmap, rmap_name)
                    if rmap == rmap_name:
                        print("rmap == rmap_name $$$$", rmap, rmap_name)
                        dict_to_test = values
                        for rmap_dict in values:
                            if seq_id is not None:
                                if type(seq_id) is not list:
                                    seq_id = [seq_id]

                                if "seq_id" in rmap_dict:
                                    rmap_seq_id = \
                                        rmap_dict["seq_id"]
                                    for _seq_id in seq_id:
                                        if _seq_id == rmap_seq_id:
                                            tmp_list.append(rmap_dict)
                        if tmp_list:
                            dict_to_test = tmp_list

                        print("dict_to_test $$$$", dict_to_test)
                        for rmap_dict in dict_to_test:
                            if "set" in rmap_dict:
                                for criteria in rmap_dict["set"].keys():
                                    if criteria not in show_bgp_json[
                                            "paths"][0]:
                                        errormsg = ("BGP attribute: {}"
                                                    " is not found in"
                                                    " cli: {} output "
                                                    "in router {}".
                                                    format(criteria,
                                                           cmd,
                                                           router))
                                        return errormsg

                                    if rmap_dict["set"][criteria] == \
                                            show_bgp_json["paths"][0][
                                                criteria]:
                                        logger.info("Verifying BGP "
                                                    "attribute {} for"
                                                    " route: {} in "
                                                    "router: {}, found"
                                                    " expected value:"
                                                    " {}".
                                                    format(criteria,
                                                           static_route,
                                                           dut,
                                                           rmap_dict[
                                                               "set"][
                                                               criteria]))
                                    else:
                                        errormsg = \
                                            ("Failed: Verifying BGP "
                                             "attribute {} for route:"
                                             " {} in router: {}, "
                                             " expected value: {} but"
                                             " found: {}".
                                             format(criteria,
                                                    static_route,
                                                    dut,
                                                    rmap_dict["set"]
                                                    [criteria],
                                                    show_bgp_json[
                                                        'paths'][
                                                        0][criteria]))
                                        return errormsg

    logger.debug("Exiting lib API: verify_bgp_attributes()")
    return True

@retry(attempts=4, wait=2, return_is_str=True, initial_wait=2)
def verify_best_path_as_per_bgp_attribute(tgen, addr_type, router, input_dict,
                                          attribute):
    """
    API is to verify best path according to BGP attributes for given routes.
    "show bgp ipv4/6 json" command will be run and verify best path according
    to shortest as-path, highest local-preference and med, lowest weight and
    route origin IGP>EGP>INCOMPLETE.

    Parameters
    ----------
    * `tgen` : topogen object
    * `addr_type` : ip type, ipv4/ipv6
    * `tgen` : topogen object
    * `attribute` : calculate best path using this attribute
    * `input_dict`: defines different routes to calculate for which route
                    best path is selected

    Usage
    -----
    # To verify best path for routes 200.50.2.0/32 and 200.60.2.0/32 from
    router r7 to router r1(DUT) as per shortest as-path attribute
    input_dict = {
        "r7": {
            "bgp": {
                "address_family": {
                    "ipv4": {
                        "unicast": {
                            "advertise_networks": [
                                {
                                    "network": "200.50.2.0/32"
                                },
                                {
                                    "network": "200.60.2.0/32"
                                }
                            ]
                        }
                    }
                }
            }
        }
    }
    attribute = "localpref"
    result = verify_best_path_as_per_bgp_attribute(tgen, "ipv4", dut, \
                         input_dict,  attribute)
    Returns
    -------
    errormsg(str) or True
    """

    logger.debug("Entering lib API: verify_best_path_as_per_bgp_attribute()")
    if router not in tgen.routers():
        return False

    rnode = tgen.routers()[router]

    command = "show bgp {} json".format(addr_type)

    sleep(5)
    logger.info("Verifying router %s RIB for best path:", router)
    sh_ip_bgp_json = run_frr_cmd(rnode, command, isjson=True)

    for route_val in input_dict.values():
        net_data = route_val["bgp"]["address_family"][addr_type]["unicast"]
        networks = net_data["advertise_networks"]
        for network in networks:
            route = network["network"]

            route_attributes = sh_ip_bgp_json["routes"][route]
            _next_hop = None
            compare = None
            attribute_dict = {}
            for route_attribute in route_attributes:
                next_hops = route_attribute["nexthops"]
                for next_hop in next_hops:
                    next_hop_ip = next_hop["ip"]
                attribute_dict[next_hop_ip] = route_attribute[attribute]

            # AS_PATH attribute
            if attribute == "aspath":
                # Find next_hop for the route have minimum as_path
                _next_hop = min(attribute_dict, key=lambda x: len(set(
                    attribute_dict[x])))
                compare = "SHORTEST"

            # LOCAL_PREF attribute
            elif attribute == "localpref":
                # Find next_hop for the route have highest local preference
                _next_hop = max(attribute_dict, key=(lambda k:
                                                     attribute_dict[k]))
                compare = "HIGHEST"

            # WEIGHT attribute
            elif attribute == "weight":
                # Find next_hop for the route have highest weight
                _next_hop = max(attribute_dict, key=(lambda k:
                                                     attribute_dict[k]))
                compare = "HIGHEST"

            # ORIGIN attribute
            elif attribute == "origin":
                # Find next_hop for the route have IGP as origin, -
                # - rule is IGP>EGP>INCOMPLETE
                _next_hop = [key for (key, value) in
                             attribute_dict.iteritems()
                             if value == "IGP"][0]
                compare = ""

            # MED  attribute
            elif attribute == "med":
                # Find next_hop for the route have LOWEST MED
                _next_hop = min(attribute_dict, key=(lambda k:
                                                     attribute_dict[k]))
                compare = "LOWEST"

            # Show ip route
            if addr_type == "ipv4":
                command = "show ip route json"
            else:
                command = "show ipv6 route json"

            rib_routes_json = run_frr_cmd(rnode, command, isjson=True)

            # Verifying output dictionary rib_routes_json is not empty
            if not bool(rib_routes_json):
                errormsg = "No route found in RIB of router {}..". \
                    format(router)
                return errormsg

            st_found = False
            nh_found = False
            # Find best is installed in RIB
            if route in rib_routes_json:
                st_found = True
                # Verify next_hop in rib_routes_json
                if rib_routes_json[route][0]["nexthops"][0]["ip"] in \
                        attribute_dict:
                    nh_found = True
                else:
                    errormsg = "Incorrect Nexthop for BGP route {} in " \
                               "RIB of router {}, Expected: {}, Found:" \
                               " {}\n".format(route, router,
                                              rib_routes_json[route][0][
                                                  "nexthops"][0]["ip"],
                                              _next_hop)
                    return errormsg

            if st_found and nh_found:
                logger.info(
                    "Best path for prefix: %s with next_hop: %s is "
                    "installed according to %s %s: (%s) in RIB of "
                    "router %s", route, _next_hop, compare,
                    attribute, attribute_dict[_next_hop], router)

    logger.debug("Exiting lib API: verify_best_path_as_per_bgp_attribute()")
    return True


def verify_best_path_as_per_admin_distance(tgen, addr_type, router, input_dict,
                                           attribute):
    """
    API is to verify best path according to admin distance for given
    route. "show ip/ipv6 route json" command will be run and verify
    best path accoring to shortest admin distanc.

    Parameters
    ----------
    * `addr_type` : ip type, ipv4/ipv6
    * `dut`: Device Under Test
    * `tgen` : topogen object
    * `attribute` : calculate best path using admin distance
    * `input_dict`: defines different routes with different admin distance
                    to calculate for which route best path is selected
    Usage
    -----
    # To verify best path for route 200.50.2.0/32 from  router r2 to
    router r1(DUT) as per shortest admin distance which is 60.
    input_dict = {
        "r2": {
            "static_routes": [{"network": "200.50.2.0/32", \
                 "admin_distance": 80, "next_hop": "10.0.0.14"},
                              {"network": "200.50.2.0/32", \
                 "admin_distance": 60, "next_hop": "10.0.0.18"}]
        }}
    attribute = "localpref"
    result = verify_best_path_as_per_admin_distance(tgen, "ipv4", dut, \
                        input_dict, attribute):
    Returns
    -------
    errormsg(str) or True
    """

    logger.debug("Entering lib API: verify_best_path_as_per_admin_distance()")
    router_list = tgen.routers()
    if router not in router_list:
        return False

    rnode = tgen.routers()[router]

    sleep(5)
    logger.info("Verifying router %s RIB for best path:", router)

    # Show ip route cmd
    if addr_type == "ipv4":
        command = "show ip route json"
    else:
        command = "show ipv6 route json"

    for routes_from_router in input_dict.keys():
        sh_ip_route_json = router_list[routes_from_router].vtysh_cmd(
            command, isjson=True)
        networks = input_dict[routes_from_router]["static_routes"]
        for network in networks:
            route = network["network"]

            route_attributes = sh_ip_route_json[route]
            _next_hop = None
            compare = None
            attribute_dict = {}
            for route_attribute in route_attributes:
                next_hops = route_attribute["nexthops"]
                for next_hop in next_hops:
                    next_hop_ip = next_hop["ip"]
                attribute_dict[next_hop_ip] = route_attribute["distance"]

            # Find next_hop for the route have LOWEST Admin Distance
            _next_hop = min(attribute_dict, key=(lambda k:
                                                 attribute_dict[k]))
            compare = "LOWEST"

        # Show ip route
        rib_routes_json = run_frr_cmd(rnode, command, isjson=True)

        # Verifying output dictionary rib_routes_json is not empty
        if not bool(rib_routes_json):
            errormsg = "No route found in RIB of router {}..".format(router)
            return errormsg

        st_found = False
        nh_found = False
        # Find best is installed in RIB
        if route in rib_routes_json:
            st_found = True
            # Verify next_hop in rib_routes_json
            if rib_routes_json[route][0]["nexthops"][0]["ip"] == \
                    _next_hop:
                nh_found = True
            else:
                errormsg = ("Nexthop {} is Missing for BGP route {}"
                            " in RIB of router {}\n".format(_next_hop,
                                                            route, router))
                return errormsg

        if st_found and nh_found:
            logger.info("Best path for prefix: %s is installed according"
                        " to %s %s: (%s) in RIB of router %s", route,
                        compare, attribute,
                        attribute_dict[_next_hop], router)

    logger.info(
        "Exiting lib API: verify_best_path_as_per_admin_distance()")
    return True
